@@ -5,31 +5,31 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: albums.py 2900 2006-02-26 21:29:13Z piman $
+# $Id: albums.py 3723 2006-07-30 18:22:32Z piman $
 
 import os
-import sys
-import gobject, gtk, pango
+
+import gobject
+import gtk
+import pango
+
 import config
 import const
 import qltk
 import util
-import stock
-from qltk.completion import EntryWordCompletion
-from qltk.views import AllTreeView
-from qltk.entry import ValidatingEntry
-from qltk.ccb import ConfigCheckButton
-from qltk.textedit import PatternEditBox
-from parse import Query, XMLFromPattern
-from formats._audio import PEOPLE
-ELPOEP = list(PEOPLE); ELPOEP.reverse()
 
-if sys.version_info < (2, 4): from sets import Set as set
-from library import library
 from browsers._base import Browser
-from qltk.properties import SongProperties
-from qltk.information import Information
+from formats import PEOPLE
+from parse import Query, XMLFromPattern
+from qltk.ccb import ConfigCheckButton
+from qltk.completion import EntryWordCompletion
+from qltk.entry import ValidatingEntry
+from qltk.songsmenu import SongsMenu
+from qltk.textedit import PatternEditBox
+from qltk.views import AllTreeView
+from util import copool, tag
 
+ELPOEP = list(PEOPLE); ELPOEP.reverse()
 EMPTY = _("Songs not in an album")
 PATTERN = r"""\<b\><title|\<i\><title>\</i\>|%s>\</b\><date| (<date>)>
 \<small\><~discs|<~discs> - ><~tracks> - <~long-length>\</small\>
@@ -80,7 +80,7 @@ class Preferences(qltk.Window):
         vbox.pack_start(label, expand=False)
         vbox.pack_start(edit)
         self.__preview_pattern(edit, label)
-        f = qltk.Frame(label=_("Album Display"), bold=True, child=vbox)
+        f = qltk.Frame(_("Album Display"), child=vbox)
         self.child.pack_start(f)
 
         self.child.show_all()
@@ -93,8 +93,9 @@ class Preferences(qltk.Window):
         AlbumList.refresh_pattern(edit.text)
 
     def __preview_pattern(self, edit, label):
-        from util import tag
-        album = AlbumList._Album(tag("album"), tag("labelid"))
+        album = AlbumList._Album(
+            util.tag("album"), util.tag("labelid"),
+            util.tag("musicbrainz_albumid"))
         album.date = "2004-10-31"
         album.length = 6319
         album.discs = 2
@@ -117,59 +118,59 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
     __gsignals__ = Browser.__gsignals__
     __model = None
 
-    def init(klass, watcher):
-        pattern_fn = os.path.join(const.DIR, "album_pattern")
+    name = _("Album List")
+    accelerated_name = _("_Album List")
+    priority = 4
+
+    @classmethod
+    def init(klass, library):
+        pattern_fn = os.path.join(const.USERDIR, "album_pattern")
         try:
             klass._Album._pattern_text = file(pattern_fn).read().rstrip()
         except EnvironmentError: pass
         else:
             klass._Album._pattern = XMLFromPattern(klass._Album._pattern_text)
-    init = classmethod(init)
 
+    @classmethod
     def toggle_covers(klass):
         on = config.getboolean("browsers", "album_covers")
         for albumlist in klass.instances():
             albumlist.__cover_column.set_visible(on)
-    toggle_covers = classmethod(toggle_covers)
 
+    @classmethod
     def refresh_pattern(klass, pattern_text):
         if pattern_text == klass._Album._pattern_text: return
         klass._Album._pattern_text = pattern_text
         klass._Album._pattern = XMLFromPattern(pattern_text)
-        for row in klass.__model:
-            album = row[0]
-            if album is not None:
-                album.markup = album._pattern % album
-                klass.__model.row_changed(row.path, row.iter)
-        pattern_fn = os.path.join(const.DIR, "album_pattern")
+        for album in [row[0] for row in klass.__model if row[0] is not None]:
+            album.finalize(cover=False)
+        pattern_fn = os.path.join(const.USERDIR, "album_pattern")
         f = file(pattern_fn, "w")
         f.write(pattern_text  + "\n")
         f.close()
-    refresh_pattern = classmethod(refresh_pattern)
 
-    def _init_model(klass, watcher):
+    @classmethod
+    def _init_model(klass, library):
         klass.__model = model = klass._AlbumStore(object)
-        watcher.connect('removed', klass.__remove_songs, model)
-        watcher.connect('changed', klass.__changed_songs, model)
-        watcher.connect('added', klass.__add_songs, model)
-        klass.__add_songs(watcher, library.values(), model)
+        library.connect('removed', klass.__remove_songs, model)
+        library.connect('changed', klass.__changed_songs, model)
+        library.connect('added', klass.__add_songs, model)
+        klass.__add_songs(library, library.values(), model)
         model.append(row=[None])
-    _init_model = classmethod(_init_model)
 
-    def __changed_songs(klass, watcher, changed, model):
-        changed = filter(lambda x: x.get("~filename") in library, changed)
-        if not changed: return
-        to_update = klass.__remove_songs(watcher, changed, model, False)
-        to_update.update(klass.__add_songs(watcher, changed, model, False))
+    @classmethod
+    def __changed_songs(klass, library, changed, model):
+        to_update = klass.__remove_songs(library, changed, model, False)
+        to_update.update(klass.__add_songs(library, changed, model, False))
         klass.__update(to_update, model)
-    __changed_songs = classmethod(__changed_songs)
 
+    @classmethod
     def __update(klass, changed, model):
         to_change = []
         to_remove = []
         for row in model:
             album = row[0]
-            if album is not None and album.title in changed:
+            if album is not None and album.key in changed:
                 if album.songs:
                     to_change.append((row.path, row.iter))
                     album.finalize()
@@ -178,61 +179,61 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                     album._model = album._iter = None
         if to_change: map(model.row_changed, *zip(*to_change))
         if to_remove: map(model.remove, to_remove)
-    __update = classmethod(__update)
 
-    def __remove_songs(klass, watcher, removed, model, update=True):
-        albums = model.get_albums()
+    @classmethod
+    def __remove_songs(klass, library, removed, model, update=True):
         changed = set()
-        for album in albums.itervalues():
-            if True in map(album.remove, removed): changed.add(album.title)
+        for row in model:
+            album = row[0]
+            if album is not None and True in map(album.remove, removed):
+                changed.add(album.key)
         if update: klass.__update(changed, model)
         else: return changed
-    __remove_songs = classmethod(__remove_songs)
 
-    def __add_songs(klass, watcher, added, model, update=True):
+    @classmethod
+    def __add_songs(klass, library, added, model, update=True):
         albums = model.get_albums()
         changed = set()
         new = []
         for song in added:
             labelid = song.get("labelid", "")
+            mbid = song.get("musicbrainz_albumid", "")
             for alb in song("album").split("\n"):
-                key = alb + "\u0000" + labelid
+                key = song.album_key
                 if key not in albums:
-                    albums[key] = klass._Album(alb, labelid)
+                    albums[key] = klass._Album(alb, labelid, mbid)
                     new.append(albums[key])
                 albums[key].songs.add(song)
-                changed.add(alb)
+                changed.add(key)
         for album in new:
-            album._model = model
-            album._iter = model.append(row=[album])
+            model.append(row=[album])
         if update: klass.__update(changed, model)
         else: return changed
-    __add_songs = classmethod(__add_songs)
 
     # Something like an AudioFile, but for a whole album.
     class _Album(object):
-        __covers = {}
         __pending_covers = []
 
         _pattern_text = PATTERN
         _pattern = XMLFromPattern(PATTERN)
 
-        def clear_cache(klass): klass.__covers.clear()
-        clear_cache = classmethod(clear_cache)
+        length = 0
+        discs = 1
+        tracks = 0
+        date = ""
+        markup = ""
 
-        def __init__(self, title, labelid):
-            self.length = 0
-            self.discs = 1
-            self.tracks = 0
-            self.date = ""
+        def __init__(self, title, labelid, mbid):
             self.people = []
-            self.title = title
-            self.labelid = labelid
             self.songs = set()
-            # cover = None indicates not gotten cover, cover = False
-            # indicates a failure to find a cover.
-            self.cover = self.__covers.get(self.title)
-            self.genre = u""
+            self.title = title
+            # The key uniquely identifies the album; this way, albums
+            # with different MBIDs or different label IDs but the same
+            # title are different, and albums with different MBIDs
+            # but the same label ID are the same (since MB uses separate
+            # MBIDs for each disc).
+            self.key = (title, labelid or mbid)
+            self.cover = None
 
         def get(self, key, default="", connector=u" - "):
             if "~" in key[1:]:
@@ -241,18 +242,13 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             elif key == "~#tracks": return self.tracks
             elif key == "~#discs": return self.discs
             elif key == "~length": return util.format_time(self.length)
+            elif key == "date": return self.date
             elif key == "~long-length":
                 return util.format_time_long(self.length)
-            elif key == "labelid": return self.labelid
-            elif key == "date": return self.date
-            elif key == "~#date":
-                try: return int(self.date[:4])
-                except (TypeError, ValueError): return 0
             elif key in ["cover", "~cover"]: return (self.cover and "y") or ""
             elif key in ["title", "album"]: return self.title
-            elif key in ["people", "artist", "artists"]:
+            elif key == "people":
                 return "\n".join(self.people)
-            elif key == "genre": return self.genre
             elif key.startswith("~#") and key[-4:-3] != ":": key += ":avg"
             elif key == "~tracks":
                 return ngettext(
@@ -264,41 +260,41 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                 else: return default
 
             if key.startswith("~#") and key[-4:-3] == ":":
-                # Using key.<func> runs the resulting list of values
+                # Using key:<func> runs the resulting list of values
                 # through the function before returning it.
                 func = key[-3:]
                 key = key[:-4]
                 func = {"max": max, "min": min, "sum": sum,
                         "avg": lambda s: float(sum(s)) / len(s)}.get(func)
                 if func: return func([song(key, 0) for song in self.songs])
-            return default
+
+            # Otherwise, if the tag isn't one provided by the album
+            # object, look in songs for it.
+            values = set()
+            for song in self.songs: values.update(song.list(key))
+            value = u"\n".join(list(values))
+            return value or default
 
         __call__ = get
-        def comma(self, *args): return self.get(*args).replace("\n", ", ")
+        def comma(self, *args):
+            return self.get(*args).replace("\n", ", ")
 
         # All songs added, cache info.
-        def finalize(self):
+        def finalize(self, cover=True):
             self.tracks = len(self.songs)
             self.length = 0
             people = {}
-            genre = set()
             for song in self.songs:
                 # Rank people by "relevance" -- artists before composers
                 # before performers, then by number of appearances.
                 for w, key in enumerate(ELPOEP):
                     for person in song.list(key):
                         people[person] = people.get(person, 0) - 1000 ** w
-                genre.update(song("genre").split("\n"))
 
                 self.discs = max(self.discs, song("~#disc", 0))
                 self.length += song.get("~#length", 0)
 
-            self.genre = "\n".join(filter(None, genre))
-            self.people = [(num, person) for (person, num) in people.items()]
-            self.people.sort()
-            self.people = [person for (num, person) in self.people[:100]]
-            self.__long_length = util.format_time_long(self.length)
-            self.__length = util.format_time(self.length)
+            self.people = sorted(people.keys(), key=people.__getitem__)[:100]
 
             if not self.title:
                 self.date = ""
@@ -306,44 +302,13 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             else: self.date = song.comma("date")
 
             self.markup = self._pattern % self
-
-            if self.title and self.cover is None:
-                self.cover = False
-                if not self.__pending_covers: gobject.idle_add(
-                    self.__get_covers, priority=gobject.PRIORITY_LOW)
-                self.__pending_covers.append([self.__get_cover, song])
+            self._model[self._iter][0] = self
 
         def remove(self, song):
             try: self.songs.remove(song)
             except KeyError: return False
             else: return True
 
-        def __get_covers(self):
-            try: get, song = self.__pending_covers.pop()
-            except IndexError: return
-            get(song)
-            gobject.idle_add(self.__get_covers, priority=gobject.PRIORITY_LOW)
-
-        def __get_cover(self, song):
-            if self._iter is None: return
-            cover = song.find_cover()
-            if cover is not None:
-                try:
-                    cover = gtk.gdk.pixbuf_new_from_file_at_size(
-                        cover.name, 48, 48)
-                except: pass
-                else:
-                    # add a black outline
-                    w, h = cover.get_width(), cover.get_height()
-                    newcover = gtk.gdk.Pixbuf(
-                        gtk.gdk.COLORSPACE_RGB, True, 8, w + 2, h + 2)
-                    newcover.fill(0x000000ff)
-                    cover.copy_area(0, 0, w, h, newcover, 1, 1)
-                    self.cover = newcover
-                    self.__covers[self.title] = newcover
-                    self._model.row_changed(
-                        self._model.get_path(self._iter), self._iter)
-
     # An auto-searching entry; it wraps is a TreeModelFilter whose parent
     # is the album list.
     class FilterEntry(ValidatingEntry):
@@ -368,7 +333,9 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             text = self.get_text().decode('utf-8')
             if Query.is_parsable(text):
                 if not text: self.__filter = None
-                else: self.__filter = Query(text).search
+                else:
+                    self.__filter = Query(
+                        text, star=["people", "album"]).search
                 self.__refill_id = gobject.timeout_add(
                     500, self.__refilter, model)
 
@@ -410,39 +377,81 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             if (a1 and a2) is None: return cmp(a1, a2)
             elif not a1.title: return 1
             elif not a2.title: return -1
-            else: return cmp(a1.title, a2.title) or cmp(a1.labelid, a2.labelid)
+            else: return cmp(a1.key, a2.key)
 
         def __compare_artist(self, model, i1, i2):
             a1, a2 = model[i1][0], model[i2][0]
             if (a1 and a2) is None: return cmp(a1, a2)
             elif not a1.title: return 1
             elif not a2.title: return -1
+            elif not a1.people and a2.people: return 1
+            elif not a2.people and a1.people: return -1
             else: return (cmp(a1.people and a1.people[0],
                               a2.people and a2.people[0]) or
-                          cmp(a1.date, a2.date) or
-                          cmp(a1.title, a2.title) or
-                          cmp(a1.labelid, a2.labelid))
+                          cmp(a1.date or "ZZZZ", a2.date or "ZZZZ") or
+                          cmp(a1.key, a2.key))
 
         def __compare_date(self, model, i1, i2):
             a1, a2 = model[i1][0], model[i2][0]
             if (a1 and a2) is None: return cmp(a1, a2)
-            elif a1.title == "": return 1
-            elif a2.title == "": return -1
-            return (cmp(a1.date, a2.date) or
-                    cmp(a1.title, a2.title) or
-                    cmp(a1.labelid, a2.labelid))
+            elif not a1.title: return 1
+            elif not a2.title: return -1
+            elif not a1.date and a2.date: return 1
+            elif not a2.date and a1.date: return -1
+            else: return (cmp(a1.date, a2.date) or cmp(a1.key, a2.key))
 
     class _AlbumStore(gtk.ListStore):
+        __gsignals__ = { "row-changed": "override" }
+
+        def __init__(self, *args, **kwargs):
+            super(AlbumList._AlbumStore, self).__init__(*args, **kwargs)
+            self.__pending_covers = []
+
+        def do_row_changed(self, path, iter):
+            album = self[iter][0]
+            if album is None:
+                return
+            album._model = self
+            album._iter = iter
+            if album.title and album.cover is None:
+                if not self.__pending_covers:
+                    copool.add(self.__scan_covers)
+                self.__pending_covers.append(album)
+
+        def __scan_covers(self):
+            while self.__pending_covers:
+                album = self.__pending_covers.pop()
+                if album._iter is None or album.cover is not None:
+                    continue
+                song = list(album.songs)[0]
+                cover = song.find_cover()
+                if cover is not None:
+                    try:
+                        cover = gtk.gdk.pixbuf_new_from_file_at_size(
+                            cover.name, 48, 48)
+                    except StandardError:
+                        continue
+                    else:
+                        # add a black outline
+                        w, h = cover.get_width(), cover.get_height()
+                        newcover = gtk.gdk.Pixbuf(
+                            gtk.gdk.COLORSPACE_RGB, True, 8, w + 2, h + 2)
+                        newcover.fill(0x000000ff)
+                        cover.copy_area(0, 0, w, h, newcover, 1, 1)
+                        album.cover = newcover
+                        self[album._iter][0] = album
+                yield True
+
         def get_albums(self):
             albums = [row[0] for row in self]
             try: albums.remove(None)
             except ValueError: pass
-            return dict([(a.title + "\u0000" + a.labelid, a) for a in albums])
+            return dict([(a.key, a) for a in albums])
 
-    def __init__(self, watcher, player):
-        gtk.VBox.__init__(self, spacing=6)
+    def __init__(self, library, player):
+        super(AlbumList, self).__init__(spacing=6)
         self._register_instance()
-        if self.__model is None: AlbumList._init_model(watcher)
+        if self.__model is None: AlbumList._init_model(library)
         self.__save = bool(player)
 
         sw = gtk.ScrolledWindow()
@@ -486,9 +495,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
 
         view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         view.set_rules_hint(True)
+        view.set_search_equal_func(self.__search_func)
+        view.set_search_column(0)
         sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
         sw.add(view)
         e = self.FilterEntry(model_filter)
+        hb2 = gtk.HBox()
+        hb2.pack_start(e)
+        hb2.pack_start(qltk.ClearButton(e), expand=False)
 
         if player: view.connect('row-activated', self.__play_selection, player)
         view.get_selection().connect('changed', self.__selection_changed, e)
@@ -498,11 +512,11 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         view.drag_source_set(
             gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
         view.connect("drag-data-get", self.__drag_data_get)
-        view.connect_object('popup-menu', self.__popup, view, watcher)
+        view.connect_object('popup-menu', self.__popup, view, library)
 
         hb = gtk.HBox(spacing=6)
         hb.pack_start(self.SortCombo(model_sort), expand=False)
-        hb.pack_start(e)
+        hb.pack_start(hb2)
         prefs = gtk.Button()
         prefs.add(
             gtk.image_new_from_stock(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
@@ -512,59 +526,33 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         self.pack_start(sw, expand=True)
         view.set_model(model_filter)
         self.show_all()
+
+    def __search_func(self, model, column, key, iter):
+        try: value = model[iter][0].title
+        except AttributeError: return True
+        else:
+            key = key.decode('utf-8')
+            return not (value.startswith(key) or value.lower().startswith(key))
         
-    def __popup(self, view, watcher):
-        menu = gtk.Menu()
-        button = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
-        props = gtk.ImageMenuItem(stock.EDIT_TAGS)
-        info = gtk.ImageMenuItem(gtk.STOCK_INFO)
-        queue = gtk.ImageMenuItem(stock.ENQUEUE)
-        rem = gtk.ImageMenuItem(stock.REMOVE)
-        
-        menu.append(button)
-        menu.append(queue)
-        menu.append(rem)
-        menu.append(props)
-        menu.append(info)
-        # Build plugins Menu
+    def __popup(self, view, library):
         songs = self.__get_selected_songs(view.get_selection())
-        songs.sort()
-        
-        submenu = self.pm.create_plugins_menu(songs)
-        if submenu is not None:
-            b = gtk.ImageMenuItem(stock.PLUGINS)
-            menu.append(b)
-            b.set_submenu(submenu)
-        menu.show_all()
-        
+        menu = SongsMenu(library, songs)
+
+        button = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
         button.connect('activate', self.__refresh_album, view.get_selection())
-        queue.connect('activate', self.__enqueue, view)
-        rem.connect('activate', self.__remove, view.get_selection(), watcher)
-        props.connect('activate', self.__properties, view, watcher)
-        info.connect('activate', self.__information, view, watcher)
-        menu.connect_object('selection-done', gtk.Menu.destroy, menu)
-        menu.popup(None, None, None, 0, gtk.get_current_event_time())
-        return True
+        menu.prepend(gtk.SeparatorMenuItem())
+        menu.prepend(button)
+        menu.show_all()
+        return view.popup_menu(menu, 0, gtk.get_current_event_time())
 
     def __refresh_album(self, menuitem, selection):
         model, rows = selection.get_selected_rows()
         albums = [model[row][0] for row in rows]
         if None in albums:
-            for row in model:
-                if row[0]:
-                    row[0].cover = None
-                    row[0].finalize()
-        else:
-            for album in albums:
-                album.cover = None
-                album.finalize()
-
-    def __remove(self, menuitem, selection, watcher):
-        songs = self.__get_selected_songs(selection)
-        if songs:
-            map(library.remove, songs)
-            watcher.removed(songs)
-            selection.unselect_all()
+            albums = [model[row][0] for row in model]
+        for album in albums:
+            album.cover = None
+            album.finalize()
 
     def __get_selected_albums(self, selection):
         model, rows = selection.get_selected_rows()
@@ -577,38 +565,24 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         model, rows = selection.get_selected_rows()
         if not model or not rows: return []
         albums = [model[row][0] for row in rows]
-        if None in albums: albums = [row[0] for row in model if row[0]]
-        songs = set()
-        map(songs.update, [album.songs for album in albums])
-        return list(songs)
-
-    def __properties(self, activator, view, watcher):
-        songs = self.__get_selected_songs(view.get_selection())
-        songs.sort()
-        if songs: SongProperties(watcher, songs)
-
-    def __information(self, activator, view, watcher):
-        songs = self.__get_selected_songs(view.get_selection())
-        songs.sort()
-        if songs: Information(watcher, songs)
+        if None in albums:
+            albums = [row[0] for row in model if row[0]]
+        # Sort first by how the albums appear in the model itself,
+        # then within the album using the default order.
+        songs = []
+        for album in albums:
+            songs.extend(sorted(album.songs))
+        return songs
 
     def __drag_data_get(self, view, ctx, sel, tid, etime):
         songs = self.__get_selected_songs(view.get_selection())
-        songs.sort()
         if tid == 1:
             filenames = [song("~filename") for song in songs]
             sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
         else: sel.set_uris([song("~uri") for song in songs])
         
-    def __enqueue(self, item, view):
-        songs = self.__get_selected_songs(view.get_selection())
-        songs.sort()
-        from widgets import main
-        main.playlist.enqueue(songs)
-
     def __play_selection(self, view, indices, col, player):
         player.reset()
-        player.next()
 
     def __preferences(self, button):
         try: prefs = AlbumList.__prefs_win
@@ -631,7 +605,9 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         for row in model:
             if row[0] is not None and row[0].title in values:
                 selection.select_path(row.path)
-                first = first or row.path[0]
+                if first is None:
+                    view.set_cursor(row.path)
+                    first = row.path[0]
         if first:
             view.scroll_to_cell(first, use_align=True, row_align=0.5)
 
@@ -641,6 +617,12 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
     def can_filter(self, key):
         return (key == "album")
 
+    def list(self, key):
+        assert (key == "album")
+        view = self.get_children()[1].child
+        model = view.get_model()
+        return [row[0].title for row in model if row[0]]
+
     def restore(self):
         albums = config.get("browsers", "albums").split("\n")
         view = self.get_children()[1].child
@@ -665,8 +647,9 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         view = self.get_children()[1].child
         model = view.get_model()
         values = song.list("album")
+        album_key = song.album_key
         for row in model:
-            if row[0] is not None and row[0].title in values:
+            if row[0] is not None and row[0].key == album_key:
                 view.scroll_to_cell(row.path[0], use_align=True, row_align=0.5)
                 break
 
@@ -685,6 +668,4 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                     confval = "\n" + confval[:-1]
                 config.set("browsers", "albums", confval)
 
-gobject.type_register(AlbumList)
-
-browsers = [(4, _("_Album List"), AlbumList, True)]
+browsers = [AlbumList]
