@@ -5,26 +5,27 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: paned.py 2760 2006-01-26 06:26:59Z piman $
+# $Id: paned.py 3634 2006-07-14 19:38:41Z piman $
+
+import gobject
+import gtk
+import pango
 
-import sys
-import gobject, gtk, pango
-import stock
 import config
 import qltk
 import util
 
-if sys.version_info < (2, 4): from sets import Set as set
-from library import library
 from browsers._base import Browser
-from qltk.songlist import SongList
-from qltk.views import AllTreeView
-from qltk.entry import ValidatingEntry
-from qltk.information import Information
-from qltk.properties import SongProperties
-from qltk.tagscombobox import TagsComboBoxEntry
-from util import tag
+from formats import PEOPLE
 from parse import Query
+from qltk.entry import ValidatingEntry
+from qltk.songlist import SongList
+from qltk.songsmenu import SongsMenu
+from qltk.tagscombobox import TagsComboBoxEntry
+from qltk.views import AllTreeView
+from util import tag
+
+UNKNOWN = "<b>%s</b>" % _("Unknown")
 
 class Preferences(qltk.Window):
     def __init__(self, *args, **kwargs):
@@ -139,6 +140,10 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
     __gsignals__ = Browser.__gsignals__
     expand = qltk.RVPaned
 
+    name = _("Paned Browser")
+    accelerated_name = _("_Paned Browser")
+    priority = 3
+
     __prefs_window = None
 
     def set_all_panes(klass):
@@ -149,7 +154,15 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
         __render = gtk.CellRendererText()
         __render.set_property('ellipsize', pango.ELLIPSIZE_END)
 
-        def __init__(self, mytag, next):
+        __render_count = gtk.CellRendererText()
+        __render_count.set_property('xalign', 1.0)
+
+        def __count_cdf(column, cell, model, iter):
+            try: cell.set_property('text', "(%d)" % (len(model[iter][1])))
+            except TypeError: cell.set_property('text', '')
+        __count_cdf = staticmethod(__count_cdf)
+
+        def __init__(self, mytag, next, library):
             super(PanedBrowser.Pane, self).__init__()
             self.tags = util.tagsplit(mytag)
             self.__next = next
@@ -159,6 +172,10 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
             column = gtk.TreeViewColumn(tag(mytag), self.__render, markup=0)
             column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             column.set_fixed_width(50)
+
+            column.pack_start(self.__render_count, expand=False)
+            column.set_cell_data_func(self.__render_count, self.__count_cdf)
+
             self.append_column(column)
             self.set_model(model)
 
@@ -166,34 +183,12 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
             selection.set_mode(gtk.SELECTION_MULTIPLE)
             self.__sig = selection.connect('changed', self.__changed)
             self.connect_object('destroy', self.__destroy, model)
-            self.connect('popup-menu', self.__popup_menu)
+            self.connect('popup-menu', self.__popup_menu, library)
 
-        def __Menu(self):
-            from widgets import main, watcher
-
-            menu = gtk.Menu()
-            songs = self.__get_songs()
-            songs.sort()
-
-            enqueue = gtk.ImageMenuItem(stock.ENQUEUE)
-            enqueue.connect_object('activate', main.playlist.enqueue, songs)
-            menu.append(enqueue)
-
-            props = gtk.ImageMenuItem(stock.EDIT_TAGS)
-            props.connect_object('activate', SongProperties, watcher, songs)
-            menu.append(props)
-
-            info = gtk.ImageMenuItem(gtk.STOCK_INFO)
-            info.connect_object('activate', Information, watcher, songs)
-            menu.append(info)
+        def __popup_menu(self, view, library):
+            menu = SongsMenu(library, sorted(self.__get_songs()))
             menu.show_all()
-            menu.connect('selection-done', lambda m: m.destroy())
-            return menu
-
-        def __popup_menu(self, view):
-            self.__Menu().popup(
-                None, None, None, 0, gtk.get_current_event_time())
-            return True
+            return view.popup_menu(menu, 0, gtk.get_current_event_time())
 
         def __destroy(self, model):
             self.set_model(None)
@@ -205,47 +200,89 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
                 self.scroll_to_cell(rows[0][0], use_align=True, row_align=0.5)
             self.__next.fill(self.__get_songs())
 
-        def __removed(self, watcher, songs):
+        def _remove(self, songs, remove_if_empty=True):
+            self.inhibit()
             model = self.get_model()
             to_remove = []
             for row in model:
                 data = row[1]
+                if data is None: continue
                 for song in songs:
                     if song in data: data.remove(song)
                 if not model[row.iter][1]: to_remove.append(row.iter)
-            map(model.remove, to_remove)
+            if remove_if_empty: map(model.remove, to_remove)
+            self.uninhibit()
+
+        def _matches(self, song):
+            try: model, rows = self.get_selection().get_selected_rows()
+            except (AttributeError, TypeError): return False
+            else:
+                if not rows or model[rows[0]][1] is None: return True
+                else:
+                    for row in rows:
+                        if model[row][0] == "<":
+                            if not bool(song.list(self.__mytag)):
+                                return True
+                        else:
+                            value = util.unescape(model[row][0])
+                            if value in song.list(self.__mytag):
+                                return True
+                    else: return False
+
+        def _add(self, songs):
+            self.inhibit()
+            model = self.get_model()
+            values = {}
+            new = {}
+            for row in model:
+                value = row[0]
+                data = row[1]
+                if value[:1] != "<":
+                    value = util.unescape(value).decode('utf-8')
+                    values[value] = data
+                elif data is not None:
+                    values[""] = data
+
+            for song in songs:
+                for val in (song.list(self.__mytag) or [""]):
+                    if val in values: values[val].add(song)
+                    else:
+                        if val not in new: new[val] = set()
+                        new[val].add(song)
+
+            if new:
+                keys = sorted(new.keys())
+                if keys[0] == "":
+                    unknown = new[""]
+                    keys.pop(0)
+                else: unknown = set()
+                for row in model:
+                    if row[0][0] == "<": continue
+                    elif not keys: break
+
+                    if util.unescape(row[0]) > keys[0]:
+                        key = keys.pop(0)
+                        model.insert_before(
+                            row.iter, row=[util.escape(key), new[key]])
+                else:
+                    for key in keys:
+                        model.append(row=[util.escape(key), new[key]])
+                if unknown:
+                    model.append(row=[UNKNOWN, new[""]])
+
+                if (len(values) + len(new)) > 1 and model[(0,)][1] is not None:
+                    model.insert(0, row=["<b>%s</b>" % _("All"), None])
+            self.uninhibit()
 
         def inhibit(self): self.get_selection().handler_block(self.__sig)
         def uninhibit(self): self.get_selection().handler_unblock(self.__sig)
 
-        def fill(self, songs, inhibit=False):
+        def fill(self, songs):
             selected = self.get_selected()
-            self.inhibit()
-            values = {}
-            unknown = set()
-            for song in songs:
-                songvals = song.list(self.__mytag)
-                if songvals:
-                    for val in songvals:
-                        values.setdefault(val, set()).add(song)
-                else: unknown.add(song)
-            keys = values.keys()
-            keys.sort()
-
             model = self.get_model()
+            self.inhibit()
             model.clear()
-            for k in keys: model.append(row=[util.escape(k), values[k]])
-
-            column = self.get_columns()[0]
-            if len(model) <= 1: column.set_title(tag(self.__mytag))
-            else:
-                column.set_title("%s (%d)" % (tag(self.__mytag), len(model)))
-
-            if len(keys) + bool(unknown) > 1:
-                model.insert(0, row=["<b>%s</b>" % _("All"), songs])
-            if unknown:
-                model.append(row=["<b>%s</b>" % _("Unknown"), unknown])
-
+            self._add(songs)
             self.uninhibit()
             if selected: self.set_selected(selected, jump=True)
             else: self.set_selected(None, jump=True)
@@ -285,48 +322,71 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
 
         def __get_songs(self):
             model, rows = self.get_selection().get_selected_rows()
-            # No reason to look further if "All" is selected.
-            if rows and rows[0][0] == 0: return list(model[(0,)][1])
+            if rows and rows[0] == (0,):
+                songs = [(row[1] or set()) for row in model]
+                return reduce(set.union, songs, set())
             else:
                 songs = [model[row][1] for row in rows]
-                if len(songs) == 1: return list(songs[0])
-                else: return list(reduce(set.union, songs, set()))
+                if len(songs) == 1: return songs[0]
+                else: return reduce(set.union, songs, set())
 
-    def __init__(self, watcher, player):
+    @classmethod
+    def init(klass, library):
+        klass.__library = library
+
+    def __init__(self, library, player):
         super(PanedBrowser, self).__init__(spacing=6)
         self._register_instance()
         self.__save = player
         hb = gtk.HBox(spacing=6)
+
         hb2 = gtk.HBox(spacing=0)
         label = gtk.Label(_("_Search:"))
         label.connect('mnemonic-activate', self.__mnemonic_activate)
         search = ValidatingEntry(Query.is_valid_color)
         label.set_mnemonic_widget(search)
         label.set_use_underline(True)
-        clr = gtk.Button()
-        clr.add(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
-        clr.connect_object('clicked', search.set_text, "")
+        clear = qltk.ClearButton(search)
         hb2.pack_start(search)
-        hb2.pack_start(clr, expand=False)
+        hb2.pack_start(clear, expand=False)
         hb.pack_start(label, expand=False)
         hb.pack_start(hb2)
 
+        self.accelerators = gtk.AccelGroup()
+        keyval, mod = gtk.accelerator_parse("<control>Home")
+        self.accelerators.connect_group(keyval, mod, 0, self.__all)
+        select = gtk.Button(_("Select _All"))
+        hb.pack_start(select, expand=False)
+
         prefs = gtk.Button()
         prefs.add(
             gtk.image_new_from_stock(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
         prefs.connect('clicked', self.__preferences)
+        select.connect('clicked', self.__all)
         hb.pack_start(prefs, expand=False)
         self.pack_start(hb, expand=False)
         self.__refill_id = None
         self.__filter = None
         search.connect('changed', self.__filter_changed)
-        for s in [watcher.connect('refresh', self.__refresh),
-                  watcher.connect('removed', self.__refresh)
+        for s in [library.connect('changed', self.__changed),
+                  library.connect('added', self.__added),
+                  library.connect('removed', self.__removed)
                   ]:
-            self.connect_object('destroy', watcher.disconnect, s)
+            self.connect_object('destroy', library.disconnect, s)
+        self.connect_object('destroy', type(self).__destroy, self)
         self.refresh_panes(restore=False)
         self.show_all()
 
+    def __destroy(self):
+        self.__save = None
+
+    def __all(self, *args):
+        self.__panes[-1].inhibit()
+        for pane in self.__panes:
+            pane.set_selected(None, True)
+        self.__panes[-1].uninhibit()
+        self.activate()
+
     def __mnemonic_activate(self, label, group_cycling):
         # If our mnemonic widget already has the focus, switch to
         # the song list instead. (#254)
@@ -347,8 +407,24 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
             else: self.__filter = None
             self.__refill_id = gobject.timeout_add(500, self.activate)
 
+    def __added(self, library, songs):
+        songs = filter(self.__filter, songs)
+        for pane in self.__panes:
+            pane._add(songs)
+            songs = filter(pane._matches, songs)
+
+    def __removed(self, library, songs, remove_if_empty=True):
+        songs = filter(self.__filter, songs)
+        for pane in self.__panes:
+            pane._remove(songs, remove_if_empty)
+
+    def __changed(self, library, songs):
+        self.__removed(library, songs, False)
+        self.__added(library, songs)
+        self.__removed(library, [])
+
     def activate(self):
-        self.__panes[0].fill(filter(self.__filter, library.values()))
+        self.__panes[0].fill(filter(self.__filter, self.__library))
 
     def scroll(self, song):
         for pane in self.__panes:
@@ -366,7 +442,8 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
         self.__panes = [self]
         panes = config.get("browsers", "panes").split(); panes.reverse()
         for pane in panes:
-            self.__panes.insert(0, self.Pane(pane, self.__panes[0]))
+            self.__panes.insert(
+                0, self.Pane(pane, self.__panes[0], self.__library))
         self.__panes.pop() # remove self
 
         for pane in self.__panes:
@@ -389,10 +466,8 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
 
     def __start(self, view, indices, col):
         self.__save.reset()
-        self.__save.next()
 
     def can_filter(self, key):
-        from formats._audio import PEOPLE
         for pane in self.__panes:
             if (key in pane.tags or
                 (key in PEOPLE and "~people" in pane.tags)):
@@ -400,7 +475,6 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
         else: return False
 
     def filter(self, key, values):
-        from formats._audio import PEOPLE
         self.__panes[-1].inhibit()
         for pane in self.__panes:
             if (key in pane.tags or
@@ -410,6 +484,14 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
         self.__panes[-1].uninhibit()
         self.__panes[-1].get_selection().emit('changed')
 
+    def list(self, key):
+        for pane in self.__panes:
+            if (key in pane.tags or
+                (key in PEOPLE and "~people" in pane.tags)):
+                return [util.unescape(row[0]) for row in pane.get_model()
+                        if row[0] and not row[0].startswith("<")]
+        else: return []
+
     def save(self):
         selected = []
         for pane in self.__panes:
@@ -425,11 +507,6 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
             self.__panes[-1].uninhibit()
             self.__panes[-1].set_selected(selected[-1].split("\t"), True)
 
-    def __refresh(self, watcher, songs=None):
-        self.__panes[-1].inhibit()
-        self.activate()
-        self.__panes[-1].uninhibit()
-
     def __preferences(self, activator):
         if PanedBrowser.__prefs_window is None:
             PanedBrowser.__prefs_window = Preferences()
@@ -439,6 +516,4 @@ class PanedBrowser(gtk.VBox, Browser, util.InstanceTracker):
         if self.__save: self.save()
         self.emit('songs-selected', list(songs), None)
 
-gobject.type_register(PanedBrowser)
-
-browsers = [(3, _("_Paned Browser"), PanedBrowser, True)]
+browsers = [PanedBrowser]
