@@ -5,27 +5,32 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: playlists.py 2909 2006-03-01 18:53:49Z piman $
+# $Id: playlists.py 3689 2006-07-24 07:31:20Z piman $
 
-import os, sys
+import os
 import urllib
-import gobject, pango, gtk
-from tempfile import NamedTemporaryFile
+
+import gobject
+import gtk
+import pango
 
 import config
 import const
+import formats
 import qltk
 import util
-import formats
-from library import library
+
+from tempfile import NamedTemporaryFile
+
 from browsers._base import Browser
 from formats._audio import AudioFile
+from library import library
+from qltk.songsmenu import SongsMenu
 from qltk.views import RCMHintedTreeView
 from qltk.wlw import WaitLoadWindow
 from util.uri import URI
-if sys.version_info < (2, 4): from sets import Set as set
 
-PLAYLISTS = os.path.join(const.DIR, "playlists")
+PLAYLISTS = os.path.join(const.USERDIR, "playlists")
 if not os.path.isdir(PLAYLISTS): util.mkdir(PLAYLISTS)
 
 def ParseM3U(filename):
@@ -175,18 +180,49 @@ class Playlist(list):
         try: return cmp(self.name, other.name)
         except AttributeError: return -1
 
+class Menu(gtk.Menu):
+    def __init__(self, songs):
+        super(Menu, self).__init__()
+        i = gtk.MenuItem(_("_New Playlist"))
+        i.connect_object('activate', self.__add_to_playlist, None, songs)
+        self.append(i)
+        self.append(gtk.SeparatorMenuItem())
+        self.set_size_request(int(i.size_request()[0] * 2), -1)
+
+        for playlist in Playlists.playlists():
+            i = gtk.MenuItem(playlist.name)
+            i.child.set_ellipsize(pango.ELLIPSIZE_END)
+            i.connect_object(
+                'activate', self.__add_to_playlist, playlist, songs)
+            self.append(i)
+
+    def __add_to_playlist(playlist, songs):
+        if playlist is None:
+            if len(songs) == 1: title = songs[0].comma("title")
+            else: title = _("%(title)s and %(count)d more") % (
+                {'title':songs[0].comma("title"), 'count':len(songs) - 1})
+            playlist = Playlist.new(title)
+        playlist.extend(songs)
+        Playlists.changed(playlist)
+    __add_to_playlist = staticmethod(__add_to_playlist)
+
 class Playlists(gtk.VBox, Browser):
     __gsignals__ = Browser.__gsignals__
     expand = qltk.RHPaned
 
-    def init(klass, watcher):
+    name = _("Playlists")
+    accelerated_name = _("_Playlists")
+    priority = 2
+    replaygain_profiles = ["track"]
+
+    def init(klass, library):
         model = klass.__lists.get_model()
         for playlist in os.listdir(PLAYLISTS):
             try: model.append(row=[Playlist(Playlist.unquote(playlist))])
             except EnvironmentError: pass
-        watcher.connect('removed', klass.__removed)
-        watcher.connect('added', klass.__added)
-        watcher.connect('changed', klass.__changed)
+        library.connect('removed', klass.__removed)
+        library.connect('added', klass.__added)
+        library.connect('changed', klass.__changed)
     init = classmethod(init)
 
     def playlists(klass): return [row[0] for row in klass.__lists]
@@ -205,18 +241,19 @@ class Playlists(gtk.VBox, Browser):
             playlist.write()
     changed = classmethod(changed)
 
-    def __removed(klass, watcher, songs):
+    def __removed(klass, library, songs):
         for playlist in klass.playlists():
             if playlist.remove_songs(songs): Playlists.changed(playlist)
     __removed = classmethod(__removed)
 
-    def __added(klass, watcher, songs):
+    def __added(klass, library, songs):
         filenames = set([song("~filename") for song in songs])
         for playlist in klass.playlists():
-            if playlist.add_songs(filenames): Playlists.changed(playlist)
+            if playlist.add_songs(filenames):
+                Playlists.changed(playlist)
     __added = classmethod(__added)
 
-    def __changed(klass, watcher, songs):
+    def __changed(klass, library, songs):
         for playlist in klass.playlists():
             for song in songs:
                 if song in playlist:
@@ -229,21 +266,22 @@ class Playlists(gtk.VBox, Browser):
         render.set_property('markup', render.markup)
     cell_data = staticmethod(cell_data)
 
-    def Menu(self, songs, songlist):
+    def Menu(self, songs, songlist, library):
+        menu = super(Playlists, self).Menu(songs, songlist, library)
         model, rows = songlist.get_selection().get_selected_rows()
         iters = map(model.get_iter, rows)
-        m = gtk.Menu()
         i = qltk.MenuItem(_("_Remove from Playlist"), gtk.STOCK_REMOVE)
         i.connect_object('activate', self.__remove, iters, model)
         i.set_sensitive(bool(self.__view.get_selection().get_selected()[1]))
-        m.append(i)
-        return m
+        menu.preseparate()
+        menu.prepend(i)
+        return menu
 
     __lists = gtk.TreeModelSort(gtk.ListStore(object))
     __lists.set_default_sort_func(lambda m, a, b: cmp(m[a][0], m[b][0]))
 
-    def __init__(self, watcher, player):
-        gtk.VBox.__init__(self, spacing=6)
+    def __init__(self, library, player):
+        super(Playlists, self).__init__(spacing=6)
         self.__main = bool(player)
         self.__view = view = RCMHintedTreeView()
         self.__render = render = gtk.CellRendererText()
@@ -265,21 +303,21 @@ class Playlists(gtk.VBox, Browser):
         newpl = gtk.Button(stock=gtk.STOCK_NEW)
         newpl.connect('clicked', self.__new_playlist)
         importpl = qltk.Button(_("_Import"), gtk.STOCK_ADD)
-        importpl.connect('clicked', self.__import, watcher)
+        importpl.connect('clicked', self.__import, library)
         hb = gtk.HBox(spacing=6)
         hb.set_homogeneous(True)
         hb.pack_start(newpl)
         hb.pack_start(importpl)
         self.pack_start(hb, expand=False)
 
-        view.connect('popup-menu', self.__popup_menu)
+        view.connect('popup-menu', self.__popup_menu, library)
 
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 0),
                    ("text/uri-list", 0, 1),
                    ("text/x-moz-url", 0, 2)]
         view.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,
                            gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_DEFAULT)
-        view.connect('drag-data-received', self.__drag_data_received, watcher)
+        view.connect('drag-data-received', self.__drag_data_received, library)
         view.connect('drag-motion', self.__drag_motion)
         view.connect('drag-leave', self.__drag_leave)
         if player: view.connect('row-activated', self.__play, player)
@@ -305,11 +343,12 @@ class Playlists(gtk.VBox, Browser):
 
     def __play(self, view, path, column, player):
         player.reset()
-        player.next()
 
     def __check_current(self, model, path, iter):
         model, citer = self.__view.get_selection().get_selected()
-        if citer and model.get_path(citer) == path: self.activate(resort=False)
+        if citer and model.get_path(citer) == path:
+            songlist = qltk.get_top_parent(self).songlist
+            self.activate(resort=not songlist.is_sorted())
 
     def __drag_motion(self, view, ctx, x, y, time):
         if "text/x-quodlibet-songs" in ctx.targets:
@@ -338,7 +377,7 @@ class Playlists(gtk.VBox, Browser):
             Playlists.changed(playlist)
             self.activate()
 
-    def __drag_data_received(self, view, ctx, x, y, sel, tid, etime, watcher):
+    def __drag_data_received(self, view, ctx, x, y, sel, tid, etime, library):
         # TreeModelSort doesn't support GtkTreeDragDestDrop.
         view.emit_stop_by_name('drag-data-received')
         model = view.get_model()
@@ -373,7 +412,7 @@ class Playlists(gtk.VBox, Browser):
             elif uri.lower().endswith('.m3u'): playlist = ParseM3U(f.name)
             else: playlist = None
             if playlist:
-                watcher.added(filter(library.add_song, playlist))
+                library.add_filename(playlist)
                 if name: playlist.rename(name)
                 Playlists.changed(playlist)
                 ctx.finish(True, False, etime)
@@ -392,14 +431,22 @@ class Playlists(gtk.VBox, Browser):
             if row[0] is playlist:
                 view.get_selection().select_iter(row.iter)
 
-    def __popup_menu(self, view):
-        self.__menu(view).popup(
-            None, None, None, 0, gtk.get_current_event_time())
-        return True
-
-    def __menu(self, view):
+    def __popup_menu(self, view, library):
         model, iter = view.get_selection().get_selected()
-        menu = gtk.Menu()
+        if iter is None:
+            return
+        songs = list(model[iter][0])
+        menu = SongsMenu(library, songs, playlists=False, remove=False)
+        menu.preseparate()
+
+        rem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
+        def remove(model, iter):
+            model[iter][0].delete()
+            model.get_model().remove(
+                model.convert_iter_to_child_iter(None, iter))
+        rem.connect_object('activate', remove, model, iter)
+        menu.prepend(rem)
+
         ren = qltk.MenuItem(_("_Rename"), gtk.STOCK_EDIT)
         keyval, mod = gtk.accelerator_parse("F2")
         ren.add_accelerator(
@@ -408,18 +455,10 @@ class Playlists(gtk.VBox, Browser):
             self.__render.set_property('editable', True)
             view.set_cursor(path, view.get_columns()[0], start_editing=True)
         ren.connect_object('activate', rename, model.get_path(iter))
-        menu.append(ren)
+        menu.prepend(ren)
 
-        rem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
-        def remove(model, iter):
-            model[iter][0].delete()
-            model.get_model().remove(
-                model.convert_iter_to_child_iter(None, iter))
-        rem.connect_object('activate', remove, model, iter)
-        menu.append(rem)
         menu.show_all()
-        menu.connect('selection-done', lambda m: m.destroy())
-        return menu
+        return view.popup_menu(menu, 0, gtk.get_current_event_time())
 
     def activate(self, widget=None, resort=True):
         model, iter = self.__view.get_selection().get_selected()
@@ -445,11 +484,10 @@ class Playlists(gtk.VBox, Browser):
         else: self.__lists[path] = self.__lists[path]
         render.set_property('editable', not self.__main)
 
-    def __import(self, activator, watcher):
+    def __import(self, activator, library):
         filt = lambda fn: fn.endswith(".pls") or fn.endswith(".m3u")
         from qltk.chooser import FileChooser
-        chooser = FileChooser(
-            self, _("Import Playlist"), filt, os.getenv("HOME"))
+        chooser = FileChooser(self, _("Import Playlist"), filt, const.HOME)
         files = chooser.run()
         chooser.destroy()
         for filename in files:
@@ -465,7 +503,7 @@ class Playlists(gtk.VBox, Browser):
                       "and PLS formats.")).run()
                 return
             Playlists.changed(playlist)
-            watcher.added(filter(library.add_song, playlist))
+            library.add(playlist)
 
     def restore(self):
         try: name = config.get("browsers", "playlist")
@@ -487,6 +525,4 @@ class Playlists(gtk.VBox, Browser):
             gobject.idle_add(self.__select_playlist, playlist)
         Playlists.changed(playlist, refresh=False)
 
-gobject.type_register(Playlists)
-
-browsers = [(2, _("_Playlists"), Playlists, True)]
+browsers = [Playlists]
