@@ -4,7 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: _audio.py 2887 2006-02-23 06:57:15Z piman $
+# $Id: _audio.py 3799 2006-08-27 01:26:56Z piman $
 
 # Much of this code is highly optimized, because many of the functions
 # are called in tight loops. Don't change things just to make them
@@ -14,12 +14,31 @@ import os
 import shutil
 import time
 
-import util, config
+import const
+import util
+
 from util.uri import URI
 
+USEFUL_TAGS = (
+    # Ogg Vorbis spec tags
+    "title version album tracknumber artist genre performer copyright "
+    "license organization description location contact isrc date "
+
+    # Other tags we like
+    "arranger author composer conductor lyricist discnumber labelid part "
+    "website language bpm albumartist originaldate originalalbum "
+    "originalartist recordingdate"
+    ).split()
+MACHINE_TAGS = (
+    "musicbrainz_trackid musicbrainz_trmid musicbrainz_albumid "
+    "musicbrainz_albumartistid musicbrainz_artistid musicip_puid "
+    "replaygain_track_gain replaygain_album_gain "
+    "replaygain_album_peak replaygain_track_peak "
+    ).split()
 MIGRATE = ("~#playcount ~#laststarted ~#lastplayed ~#added "
-           "~#skipcount ~#rating").split()
-PEOPLE = "artist author composer performer lyricist arranger conductor".split()
+           "~#skipcount ~#rating ~bookmark").split()
+PEOPLE = ("albumartist artist author composer ~performers originalartist "
+          "lyricist arranger conductor").split()
 
 class AudioFile(dict):
     """An audio file. It looks like a dict, but implements synthetic
@@ -38,22 +57,38 @@ class AudioFile(dict):
     format = "Unknown Audio File"
 
     def __sort_key(self):
-        return (self.get("album"), self.get("labelid"),
-                self("~#disc"), self("~#track"),
-                self.get("artist"), self.get("title"),
+        return (self.get("album"),
+                self.get("labelid") or self.get("musicbrainz_albumid"),
+                self("~#disc", self.get("discnumber")),
+                self("~#track", self.get("tracknumber")),
+                self.get("artist"), self.get("musicbrainz_artistid"),
+                self.get("title"),
                 self.get("~filename"))
     sort_key = property(__sort_key)
 
+    key = property(lambda self: self["~filename"])
+    mountpoint = property(lambda self: self["~mountpoint"])
+
+    def __album_key(self):
+        return (self.get("album", ""),
+                self.get("labelid") or self.get("musicbrainz_albumid") or "")
+    album_key = property(__album_key)
+
     def __cmp__(self, other):
         if not other: return -1
         try: return cmp(self.sort_key, other.sort_key)
         except AttributeError: return -1
 
-    def __eq__(self, other):
-        """AudioFiles are equal if they have the same filename."""
+    def __hash__(self):
+        # Dicts aren't hashable by default, so we need a hash
+        # function. Previously this used ~filename. That created a
+        # situation when an object could end up in two buckets by
+        # renaming files. So now it uses identity.
+        return hash(id(self))
 
-        try: return self.get("~filename") == other.get("~filename")
-        except: return False
+    def __eq__(self, other):
+        # And to preserve Python hash rules, we need a strict __eq__.
+        return self is other
 
     def reload(self):
         """Reload an audio file from disk. The caller is responsible for
@@ -68,9 +103,6 @@ class AudioFile(dict):
         self.__init__(fn)
         self.update(saved)
 
-    def __hash__(self):
-        return hash(self["~filename"])
-
     def realkeys(self):
         """Returns a list of keys that are not internal, i.e. they don't
         have '~' in them."""
@@ -90,28 +122,39 @@ class AudioFile(dict):
         if key[:1] == "~":
             key = key[1:]
             if "~" in key:
-                parts = filter(None, map(self.__call__, util.tagsplit(key)))
-                return connector.join(parts)
-            elif key == "basename": return os.path.basename(self["~filename"])
-            elif key == "dirname": return os.path.dirname(self["~filename"])
-            elif key == "length":
-                if self.get("~#length", 0) == 0: return default
-                else: return util.format_time(self.get("~#length", 0))
-            elif key == "rating":
-                return util.format_rating(self.get("~#rating", 0))
+                return connector.join(
+                    filter(None, map(self.__call__, util.tagsplit("~" + key))))
             elif key == "#track":
                 try: return int(self["tracknumber"].split("/")[0])
                 except (ValueError, TypeError, KeyError): return default
             elif key == "#disc":
                 try: return int(self["discnumber"].split("/")[0])
                 except (ValueError, TypeError, KeyError): return default
+            elif key == "length":
+                if self.get("~#length", 0) == 0: return default
+                else: return util.format_time(self.get("~#length", 0))
+            elif key == "rating":
+                return util.format_rating(self.get("~#rating", 0))
             elif key == "people":
                 join = "\n".join
-                people = filter(None, map(self.get, PEOPLE))
+                people = filter(None, map(self.__call__, PEOPLE))
                 people = join(people).split("\n")
                 index = people.index
                 return join([person for (i,person) in enumerate(people)
                         if index(person)==i])
+            elif key == "performers":
+                values = []
+                for key in self.realkeys():
+                    if key.startswith("performer:"):
+                        role = key.split(":", 1)[1]
+                        for value in self.list(key):
+                            values.append("%s (%s)" % (value, role))
+                    values.extend(self.list("performer"))
+                return "\n".join(values)
+            elif key == "basename":
+                return os.path.basename(self["~filename"]) or self["~filename"]
+            elif key == "dirname":
+                return os.path.dirname(self["~filename"]) or self["~filename"]
             elif key == "uri":
                 try: return self["~uri"]
                 except KeyError:
@@ -131,20 +174,29 @@ class AudioFile(dict):
                 try: return int(self["discnumber"].split("/")[1])
                 except (ValueError, IndexError, TypeError, KeyError):
                     return default
+            elif key == "lyrics":
+                try: fileobj = file(self.lyric_filename, "rU")
+                except EnvironmentError: return default
+                else: return fileobj.read().decode("utf-8", "replace")
             elif key[0] == "#" and "~" + key not in self:
                 try: return int(self[key[1:]])
                 except (ValueError, TypeError, KeyError): return default
             else: return dict.get(self, "~" + key, default)
 
         elif key == "title":
-            v = dict.get(self, "title")
-            if v is None:
-                return "%s [%s]" %(
-                    os.path.basename(self["~filename"]).decode(
-                    util.fscoding, "replace"), _("Unknown"))
-            else: return v
+            title = dict.get(self, "title")
+            if title is None:
+                basename = self("~basename")
+                basename = basename.decode(const.FSCODING, "replace")
+                return "%s [%s]" % (basename, _("Unknown"))
+            else: return title
         else: return dict.get(self, key, default)
 
+    lyric_filename = property(lambda self: util.fsencode(
+        os.path.join(os.path.expanduser("~/.lyrics"),
+                     self.comma("artist").replace('/', '')[:128],
+                     self.comma("title").replace('/', '')[:128] + '.lyric')))
+
     def comma(self, key):
         """Get all values of a tag, separated by commas. Synthetic
         tags are supported, but will be slower. If the value is
@@ -323,8 +375,7 @@ class AudioFile(dict):
         try: fns = os.listdir(base)
         except EnvironmentError:  return None
         images = []
-        fns.sort()
-        for fn in fns:
+        for fn in sorted(fns):
             lfn = fn.lower()
             if lfn[-4:] in ["jpeg", ".jpg", ".png", ".gif"]:
                 score = 0
@@ -341,26 +392,63 @@ class AudioFile(dict):
             return self.get_format_cover()
         else: return None
 
-    def replay_gain(self):
-        """Return the recommended ReplayGain scale factor as a floating
-        point number, based on the current settings."""
+    def replay_gain(self, profiles):
+        """Return the recommended Replay Gain scale factor.
 
-        gain = config.getint("settings", "gain")
-        try:
-            if gain == 0: raise ValueError
-            elif gain == 2 and "replaygain_album_gain" in self:
-                db = float(self["replaygain_album_gain"].split()[0])
-                peak = float(self["replaygain_album_peak"])
-            elif gain > 0 and "replaygain_track_gain" in self:
-                db = float(self["replaygain_track_gain"].split()[0])
-                peak = float(self["replaygain_track_peak"])
-            else: raise ValueError
-            scale = 10.**(db / 20)
-            if scale * peak > 1: scale = 1.0 / peak # don't clip
-            return min(15, scale)
-        except (KeyError, ValueError):
-            return 1
+        profiles is a list of Replay Gain profile names ('album',
+        'track') to try before giving up. The special profile name
+        'none' will cause no scaling to occur.
+        """
+        for profile in profiles:
+            if profile is "none":
+                return 1.0
+            try:
+                db = float(self["replaygain_%s_gain" % profile].split()[0])
+                peak = float(self.get("replaygain_%s_peak" % profile, 1))
+            except (KeyError, ValueError):
+                continue
+            else:
+                scale = 10.**(db / 20)
+                if scale * peak > 1:
+                    scale = 1.0 / peak # don't clip
+                return min(15, scale)
+        else:
+            return 1.0
 
     def write(self):
         """Write metadata back to the file."""
         raise NotImplementedError
+
+    def __get_bookmarks(self):
+        marks = []
+        invalid = []
+        for line in self.list("~bookmark"):
+            try: time, mark = line.split(" ", 1)
+            except: invalid.append((-1, line))
+            else:
+                try: time = util.parse_time(time, None)
+                except: invalid.append((-1, line))
+                else:
+                    if time >= 0: marks.append((time, mark))
+                    else: invalid.append((-1, line))
+        marks.sort()
+        marks.extend(invalid)
+        return marks
+
+    def __set_bookmarks(self, marks):
+        result = []
+        for time, mark in marks:
+            if time < 0: raise ValueError("mark times must be positive")
+            result.append(u"%s %s" % (util.format_time(time), mark))
+        result = u"\n".join(result)
+        if result: self["~bookmark"] = result
+        elif "~bookmark" in self: del(self["~bookmark"])
+
+    bookmarks = property(
+        __get_bookmarks, __set_bookmarks,
+        doc="""Parse and return song position bookmarks, or set them.
+        Accessing this returns a copy, so song.bookmarks.append(...)
+        will not work; you need to do
+           marks = song.bookmarks
+           marks.append(...)
+           song.bookmarks = marks""")
