@@ -4,10 +4,12 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: mod.py 2807 2006-02-05 03:18:52Z piman $
+# $Id: mod.py 3684 2006-07-23 22:20:43Z piman $
 
 import os
+
 import gst
+
 from formats._audio import AudioFile
 
 extensions = [
@@ -16,12 +18,13 @@ extensions = [
 try:
     import ctypes
     _modplug = ctypes.cdll.LoadLibrary("libmodplug.so.0")
-except: extensions = []
+except (ImportError, OSError):
+    extensions = []
 else:
-    try: gst.element_factory_make("modplug")
-    except:
-        try: gst.element_factory_make("mikmod")
-        except: extensions = []
+    _modplug.ModPlug_GetName.restype = ctypes.c_char_p
+
+    if gst.registry_get_default().find_plugin("modplug") is None:
+        extensions = []
 
 class ModFile(AudioFile):
 
