@@ -4,25 +4,26 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: wavpack.py 2807 2006-02-05 03:18:52Z piman $
+# $Id: wavpack.py 3684 2006-07-23 22:20:43Z piman $
 
 import gst
+
 from formats._apev2 import APEv2File
 
+extensions = [".wv"]
 try:
     import ctypes
     _wavpack = ctypes.cdll.LoadLibrary("libwavpack.so.0")
-except: extensions = []
+except (ImportError, OSError):
+    extensions = []
 else:
-    try: gst.element_factory_make('wavpackdec')
-    except: extensions = []
-    else:
-        extensions = [".wv"]
-        _wavpack.WavpackGetSampleRate.restype = ctypes.c_uint32
-        _wavpack.WavpackGetNumSamples.restype = ctypes.c_uint32
+    _wavpack.WavpackGetSampleRate.restype = ctypes.c_uint32
+    _wavpack.WavpackGetNumSamples.restype = ctypes.c_uint32
+    if gst.registry_get_default().find_plugin("wavpack") is None:
+        extensions = []
 
 class WavpackFile(APEv2File):
-    format = "Wavpack"
+    format = "WavPack"
     
     def __init__(self, filename):
         super(WavpackFile, self).__init__(filename)
