@@ -6,6 +6,7 @@
 # date the song was added to the library
 _("added")
 _("album")
+_("album artist")
 _("arranger")
 _("artist")
 _("author")
@@ -31,6 +32,8 @@ _("label ID")
 _("language")
 # date the song was last heard on
 _("last played")
+# date the song was last started on
+_("last started")
 _("length")
 _("license")
 _("location")
@@ -40,19 +43,23 @@ _("modified")
 _("mount point")
 # alternately, "record label"
 _("organization")
+_("original release date")
+_("original artist")
+_("original album")
 # alternately, "movement", "section"
 _("part")
 _("people")
 _("performer")
 _("plays")
 _("rating")
+_("recording date")
 _("skips")
 _("title")
 _("track")
 _("version")
 _("website")
 _("year")
-# Beats per minute; I don't know if other languages use a different word.
+# beats per minute
 _("BPM")
 
 _("performers")
