@@ -5,25 +5,116 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: edittags.py 2831 2006-02-13 21:12:22Z piman $
+# $Id: edittags.py 3797 2006-08-21 01:44:26Z alextp $
 
-import os, sys
-import gtk, pango
+import traceback
+
+import gtk
+import pango
 
 import qltk
-from qltk.wlw import WritingWindow
-from qltk.views import RCMHintedTreeView
-from qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
-from massagers import Massager
 
-import const
 import config
-import util
 import formats
+import util
+import util.massagers
 
-from util import tag
+from qltk.completion import LibraryValueCompletion
+from qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
+from qltk.views import RCMHintedTreeView
+from qltk.wlw import WritingWindow
 
-import __builtin__; __builtin__.__dict__.setdefault("_", lambda a: a)
+class AudioFileGroup(dict):
+
+    class Comment(unicode):
+        complete = True
+        def __repr__(self):
+            return '%s %s' % (str(self), self.paren())
+
+        def __str__(self):
+            return util.escape(self)
+
+        def paren(self):
+            if self.shared:
+                return ngettext('missing from %d song',
+                                'missing from %d songs',
+                                self.missing) % self.missing
+            elif self.complete:
+                return ngettext('different across %d song',
+                                'different across %d songs',
+                                self.total) % self.total
+            else:
+                d = ngettext('different across %d song',
+                              'different across %d songs',
+                              self.have) % self.have
+                m = ngettext('missing from %d song',
+                              'missing from %d songs',
+                              self.missing) % self.missing
+                return ", ".join([d, m])
+
+        def safenicestr(self):
+            if self.shared and self.complete: return str(self)
+            elif self.shared:
+                return "\n".join(['%s <i>(%s)</i>' % (s, self.paren())
+                                  for s in str(self).split("\n")])
+            else: return '<i>(%s)</i>' % self.paren()
+
+    class SharedComment(Comment): shared = True
+    class UnsharedComment(Comment): shared = False
+    class PartialSharedComment(SharedComment): complete = False
+    class PartialUnsharedComment(UnsharedComment): complete = False
+
+    def realkeys(self):
+        return filter(lambda s: s and "~" not in s and "=" not in s, self)
+
+    is_file = True
+    multiple_values = True
+
+    def __init__(self, songs):
+        keys = {}
+        first = {}
+        all = {}
+        total = len(songs)
+        self.songs = songs
+
+        for song in songs:
+            self.is_file &= song.is_file
+            self.multiple_values &= song.multiple_values
+            for comment, val in song.iteritems():
+                keys[comment] = keys.get(comment, 0) + 1
+                first.setdefault(comment, val)
+                all[comment] = all.get(comment, True) and first[comment] == val
+
+        # collect comment representations
+        for comment, count in keys.iteritems():
+            if count < total:
+                if all[comment]:
+                    value = self.PartialSharedComment(first[comment])
+                else:
+                    value = self.PartialUnsharedComment(first[comment])
+            else:
+                decoded = first[comment]
+                if isinstance(decoded, str): decoded = util.decode(decoded)
+                if all[comment]: value = self.SharedComment(decoded)
+                else: value = self.UnsharedComment(decoded)
+            value.have = count
+            value.total = total
+            value.missing = total - count
+
+            self[comment] = value
+
+    def can_change(self, k=None):
+        if k is None:
+            can = True
+            for song in self.songs:
+                cantoo = song.can_change()
+                if can is True: can = cantoo
+                elif cantoo is True: pass
+                else: can = set(can) | set(cantoo)
+        else:
+            if not self.songs: return False
+            can = min([song.can_change(k) for song in self.songs])
+        return can
 
 class SplitValues(gtk.ImageMenuItem):
     tags = False
@@ -104,12 +195,13 @@ class SplitPerformer(SplitPerson):
     title = _("Split _Performer out of Artist")
 
 class AddTagDialog(gtk.Dialog):
-    def __init__(self, parent, can_change):
-        if can_change == True: can = formats.USEFUL_TAGS
-        else: can = list(can_change)
-        can.sort()
-
-        gtk.Dialog.__init__(self, _("Add a Tag"), parent)
+    def __init__(self, parent, can_change, library):
+        if can_change == True:
+            can = sorted(formats.USEFUL_TAGS)
+        else:
+            can = sorted(can_change)
+        super(AddTagDialog, self).__init__(
+            _("Add a Tag"), qltk.get_top_parent(parent))
         self.set_border_width(6)
         self.set_has_separator(False)
         self.set_resizable(False)
@@ -134,6 +226,7 @@ class AddTagDialog(gtk.Dialog):
         table.attach(self.__tag, 1, 2, 0, 1)
 
         self.__val = gtk.Entry()
+        self.__val.set_completion(LibraryValueCompletion("", library))
         label = gtk.Label()
         label.set_text(_("_Value:"))
         label.set_alignment(0.0, 0.5)
@@ -159,6 +252,17 @@ class AddTagDialog(gtk.Dialog):
         for entry in [self.__tag, self.__val]:
             entry.connect(
                 'changed', self.__validate, add, invalid, tips, valuebox)
+        self.__tag.connect('changed', self.__set_value_completion, library)
+        self.__set_value_completion(self.__tag, library)
+
+        if can_change == True:
+            self.__tag.child.connect_object(
+                'activate', gtk.Entry.grab_focus, self.__val)
+
+    def __set_value_completion(self, tag, library):
+        completion = self.__val.get_completion()
+        if completion:
+            completion.set_tag(self.__tag.tag, library)
 
     def get_tag(self):
         try: return self.__tag.tag
@@ -171,7 +275,7 @@ class AddTagDialog(gtk.Dialog):
     def __validate(self, editable, add, invalid, tips, box):
         tag = self.get_tag()
         value = self.get_value()
-        fmt = Massager.fmt.get(tag)
+        fmt = util.massagers.tags.get(tag)
         if fmt: valid = bool(fmt.validate(value))
         else: valid = True
         add.set_sensitive(valid)
@@ -185,21 +289,32 @@ class AddTagDialog(gtk.Dialog):
 
     def run(self):
         self.show()
-        try: self.__tag.child.set_activates_default(True)
-        except AttributeError: pass
         self.__val.set_activates_default(True)
         self.__tag.grab_focus()
-        return gtk.Dialog.run(self)
+        return super(AddTagDialog, self).run()
 
-TAG, VALUE, EDITED, CANEDIT, DELETED, ORIGVALUE = range(6)
+TAG, VALUE, EDITED, CANEDIT, DELETED, ORIGVALUE, RENAMED, ORIGTAG = range(8)
+
+def is_special(string):
+    return string.endswith("</i>")
+
+def is_different(string):
+    return is_special(string) and string.startswith("<i>")
+
+def is_missing(string):
+    return is_special(string) and not string.startswith("<i>")
+
+def strip_missing(string):
+    try: return string[:string.index(" <i>")]
+    except ValueError: return string
 
 class EditTags(gtk.VBox):
-    def __init__(self, parent, watcher):
+    def __init__(self, parent, library):
         super(EditTags, self).__init__(spacing=12)
         self.title = _("Edit Tags")
         self.set_border_width(12)
 
-        model = gtk.ListStore(str, str, bool, bool, bool, str)
+        model = gtk.ListStore(str, str, bool, bool, bool, str, bool, str)
         view = RCMHintedTreeView(model)
         selection = view.get_selection()
         render = gtk.CellRendererPixbuf()
@@ -227,12 +342,16 @@ class EditTags(gtk.VBox):
         column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
         render.set_property('editable', True)
         render.connect('edited', self.__edit_tag_name, model)
+        render.connect(
+            'editing-started', self.__tag_editing_started, model, library)
         view.append_column(column)
 
         render = gtk.CellRendererText()
         render.set_property('ellipsize', pango.ELLIPSIZE_END)
         render.set_property('editable', True)
         render.connect('edited', self.__edit_tag, model)
+        render.connect(
+            'editing-started', self.__value_editing_started, model, library)
         render.markup = 1
         column = gtk.TreeViewColumn(
             _('Value'), render, markup=1, editable=3, strikethrough=4)
@@ -251,7 +370,7 @@ class EditTags(gtk.VBox):
         bbox1.set_layout(gtk.BUTTONBOX_START)
         add = gtk.Button(stock=gtk.STOCK_ADD)
         add.set_focus_on_click(False)
-        add.connect('clicked', self.__add_tag, model)
+        add.connect('clicked', self.__add_tag, model, library)
         remove = gtk.Button(stock=gtk.STOCK_REMOVE)
         remove.set_focus_on_click(False)
         remove.connect('clicked', self.__remove_tag, view)
@@ -282,8 +401,7 @@ class EditTags(gtk.VBox):
             'clicked', self.__update, None, *UPDATE_ARGS)
         revert.connect_object('clicked', parent.set_pending, None)
 
-        save.connect(
-            'clicked', self.__save_files, revert, model, parent, watcher)
+        save.connect('clicked', self.__save_files, revert, model, library)
         save.connect_object('clicked', parent.set_pending, None)
         for sig in ['row-inserted', 'row-deleted', 'row-changed']:
             model.connect(sig, self.__enable_save, [save, revert])
@@ -291,10 +409,17 @@ class EditTags(gtk.VBox):
 
         view.connect('popup-menu', self.__popup_menu, parent)
         view.connect('button-press-event', self.__button_press)
+        view.connect('key-press-event', self.__view_key_press_event)
         selection.connect('changed', self.__tag_select, remove)
         selection.set_mode(gtk.SELECTION_MULTIPLE)
         self.show_all()
 
+    def __view_key_press_event(self, view, event):
+        # We can't use a real accelerator to this because it would
+        # interfere with typeahead and row editing.
+        if event.keyval == gtk.accelerator_parse("Delete")[0]:
+            self.__remove_tag(view, view)
+
     def __enable_save(self, *args):
         buttons = args[-1]
         for b in buttons: b.set_sensitive(True)
@@ -324,14 +449,14 @@ class EditTags(gtk.VBox):
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
 
+        view.ensure_popup_selection()
         model, rows = view.get_selection().get_selected_rows()
         can_change = min([model[path][CANEDIT] for path in rows])
 
         items = [SplitDisc, SplitTitle, SplitPerformer, SplitArranger,
                  SplitValues]
         items.extend(parent.plugins.EditTagsPlugins())
-        items.sort(lambda a, b:
-                   (cmp(a._order, b._order) or cmp(a.__name__, b.__name__)))
+        items.sort(key=lambda item: (item._order, item.__name__))
 
         if len(rows) == 1:
             row = model[rows[0]]
@@ -343,7 +468,7 @@ class EditTags(gtk.VBox):
 
                 try: b = Item(row[TAG], text)
                 except:
-                    import traceback; traceback.print_exc()
+                    traceback.print_exc()
                 else:
                     b.connect('activate', self.__menu_activate, view)
 
@@ -357,6 +482,9 @@ class EditTags(gtk.VBox):
 
         b = gtk.ImageMenuItem(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
         b.connect('activate', self.__remove_tag, view)
+        keyval, mod = gtk.accelerator_parse("Delete")
+        b.add_accelerator(
+            'activate', gtk.AccelGroup(), keyval, mod, gtk.ACCEL_VISIBLE)
         menu.append(b)
 
         menu.show_all()
@@ -365,48 +493,47 @@ class EditTags(gtk.VBox):
         for c in menu.get_children():
             c.set_sensitive(can_change and c.get_property('sensitive'))
         menu.connect('selection-done', lambda m: m.destroy())
-        menu.popup(None, None, None, 3, gtk.get_current_event_time())
-        return True
+        return view.popup_menu(menu, 3, gtk.get_current_event_time())
 
     def __tag_select(self, selection, remove):
         model, rows = selection.get_selected_rows()
         remove.set_sensitive(
             bool(rows and min([model[row][CANEDIT] for row in rows])))
 
-    def __add_new_tag(self, model, comment, value):
-        if (comment in self.__songinfo and not self.__songinfo.multiple_values):
+    def __add_new_tag(self, model, tag, value):
+        if (tag in self.__songinfo and not self.__songinfo.multiple_values):
             title = _("Unable to add tag")
             msg = _("Unable to add <b>%s</b>\n\nThe files currently"
                     " selected do not support multiple values."
-                    ) % util.escape(comment)
-            qltk.ErrorMessage(None, title, msg).run()
+                    ) % util.escape(tag)
+            qltk.ErrorMessage(self, title, msg).run()
             return
 
         edited = True
         edit = True
         orig = None
         deleted = False
-        iters = [row.iter for row in model if row[TAG] == comment]
-        row = [comment, util.escape(value), True, True, False, None]
+        iters = [row.iter for row in model if row[TAG] == tag]
+        row = [tag, util.escape(value), True, True, False, None, False, None]
         if len(iters): model.insert_after(iters[-1], row=row)
         else: model.append(row=row)
 
-    def __add_tag(self, activator, model):
-        add = AddTagDialog(None, self.__songinfo.can_change())
+    def __add_tag(self, activator, model, library):
+        add = AddTagDialog(self, self.__songinfo.can_change(), library)
 
         while True:
             resp = add.run()
             if resp != gtk.RESPONSE_OK: break
             tag = add.get_tag()
             value = add.get_value()
-            if tag in Massager.fmt:
-                value = Massager.fmt[tag].validate(value)
+            if tag in util.massagers.tags:
+                value = util.massagers.tags[tag].validate(value)
             if not self.__songinfo.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
                         " selected do not support editing this tag."
                         ) % util.escape(tag)
-                qltk.ErrorMessage(None, title, msg).run()
+                qltk.ErrorMessage(self, title, msg).run()
             else:
                 self.__add_new_tag(model, tag, value)
                 break
@@ -415,18 +542,21 @@ class EditTags(gtk.VBox):
 
     def __remove_tag(self, activator, view):
         model, paths = view.get_selection().get_selected_rows()
-        rows = map(model.__getitem__, paths)
+        # Since the iteration can modify path numbers, we need accurate
+        # rows (= iters) before we start.
+        rows = [model[path] for path in paths]
         for row in rows:
             if row[ORIGVALUE] is not None:
                 row[EDITED] = row[DELETED] = True
             else: model.remove(row.iter)
 
-    def __save_files(self, save, revert, model, parent, watcher):
+    def __save_files(self, save, revert, model, library):
         updated = {}
         deleted = {}
         added = {}
+        renamed = {}
         for row in model:
-            if row[EDITED] and not row[DELETED]:
+            if row[EDITED] and not (row[DELETED] or row[RENAMED]):
                 if row[ORIGVALUE] is not None:
                     updated.setdefault(row[TAG], [])
                     updated[row[TAG]].append((util.decode(row[VALUE]),
@@ -439,11 +569,17 @@ class EditTags(gtk.VBox):
                     deleted.setdefault(row[TAG], [])
                     deleted[row[TAG]].append(util.decode(row[ORIGVALUE]))
 
+            if row[EDITED] and row[RENAMED] and not row[DELETED]:
+                renamed.setdefault(row[TAG], [])
+                renamed[row[TAG]].append((util.decode(row[ORIGTAG]),
+                                         util.decode(row[VALUE])))
+                
         was_changed = []
-        win = WritingWindow(parent, len(self.__songs))
-        for song in self.__songs:
+        songs = self.__songinfo.songs
+        win = WritingWindow(self, len(songs))
+        for song in songs:
             if not song.valid() and not qltk.ConfirmAction(
-                None, _("Tag may not be accurate"),
+                self, _("Tag may not be accurate"),
                 _("<b>%s</b> changed while the program was running. "
                   "Saving without refreshing your library may "
                   "overwrite other changes to the song.\n\n"
@@ -472,80 +608,111 @@ class EditTags(gtk.VBox):
                     if song.can_change(key) and key in song:
                         song.remove(key, value)
                         changed = True
+            save_rename = []
+            for new_tag, values in renamed.iteritems():
+                for old_tag, value in values:
+                    old_tag = util.unescape(old_tag)
+                    value = util.unescape(value)
+                    if (song.can_change(new_tag) and
+                        song.can_change(old_tag) and old_tag in song):
+                        if not is_special(value):
+                            song.remove(old_tag, value)
+                            save_rename.append((new_tag, value))
+                        elif is_missing(value):
+                            value = strip_missing(value)
+                            song.remove(old_tag, value)
+                            save_rename.append((new_tag, value))
+                        else:
+                            save_rename.append((new_tag, song[old_tag]))
+                            song.remove(old_tag, None)
+                        changed = True
+            for tag, value in save_rename:
+                song.add(tag, value)
 
             if changed:
                 try: song.write()
                 except:
                     qltk.ErrorMessage(
-                        None, _("Unable to save song"),
+                        self, _("Unable to save song"),
                         _("Saving <b>%s</b> failed. The file "
                           "may be read-only, corrupted, or you "
                           "do not have permission to edit it.")%(
                         util.escape(util.fsdecode(
                         song('~basename'))))).run()
-                    watcher.reload(song)
+                    library.reload(song)
                     break
                 was_changed.append(song)
 
             if win.step(): break
 
         win.destroy()
-        watcher.changed(was_changed)
-        watcher.refresh()
+        library.changed(was_changed)
         for b in [save, revert]: b.set_sensitive(False)
 
-    def __edit_tag_name(self, renderer, path, new, model):
-        new = ' '.join(new.splitlines())
+    def __edit_tag(self, renderer, path, new_value, model):
+        new_value = ', '.join(new_value.splitlines())
         row = model[path]
-        if new == row[TAG]: return
-        elif self.__songinfo.can_change(new):
-            if new in Massager.fmt:
-                fmt = Massager.fmt[new]
-                value = fmt.validate(util.unescape(row[VALUE]))
-                if not value:
-                    qltk.WarningMessage(
-                        None, _("Invalid value"), _("Invalid value") +
-                        (": <b>%s</b>\n\n%s" % (row[VALUE], fmt.error))).run()
+        if row[TAG] in util.massagers.tags:
+            fmt = util.massagers.tags[row[TAG]]
+            new_validated = fmt.validate(new_value)
+            if not new_validated:
+                qltk.WarningMessage(
+                    self, _("Invalid value"),
+                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") %{
+                    "value": new_value, "error": fmt.error}).run()
                 return
-            else:
-                value = row[VALUE]
-                idx = value.find('<i>')
-                if idx == 0:
-                    title = _("Unable to retag multiple values")
-                    msg = _("Changing the name of a tag with multiple "
-                            "values is not supported.")
-                    qltk.ErrorMessage(None, title, msg).run()
-                    return
-                elif idx >= 0: value = value[:idx].strip()
-                value = util.unescape(value)
+            else: new_value = new_validated
+        if row[VALUE].replace('<i>','').replace('</i>','') != new_value:
+            row[VALUE] = util.escape(new_value)
+            row[EDITED] = True
+            row[DELETED] = False
 
-            if row[ORIGVALUE] is None: row[0] = new
-            else:
-                row[DELETED] = row[EDITED] = True
-                self.__add_new_tag(model, new, value)
+    def __edit_tag_name(self, renderer, path, new_tag, model):
+        new_tag = ' '.join(new_tag.splitlines()).lower()
+        row = model[path]
+        if new_tag == row[TAG]:
+            return
         elif not self.__songinfo.can_change(row[TAG]):
+            # Can't remove the old tag.
             title = _("Invalid tag")
             msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
                     " selected do not support editing this tag."
-                    ) % util.escape(comment)
-            qltk.ErrorMessage(None, title, msg).run()            
+                    ) % util.escape(row[TAG])
+            qltk.ErrorMessage(self, title, msg).run()            
+        elif not self.__songinfo.can_change(new_tag):
+            # Can't add the new tag.
+            title = _("Invalid tag")
+            msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
+                    " selected do not support editing this tag."
+                    ) % util.escape(new_tag)
+            qltk.ErrorMessage(self, title, msg).run()
+        else:
+            if new_tag in util.massagers.tags:
+                fmt = util.massagers.tags[new_tag]
+                value = fmt.validate(util.unescape(row[VALUE]))
+                if not value:
+                    qltk.WarningMessage(
+                        self, _("Invalid value"),
+                        _("Invalid value: <b>%(value)s</b>\n\n%(error)s") %{
+                        "value": row[VALUE], "error": fmt.error}).run()
+                    return
+            else:
+                value = row[VALUE]
+                idx = value.find('<i>')
+                value = util.unescape(value)
 
-    def __edit_tag(self, renderer, path, new, model):
-        new = ', '.join(new.splitlines())
-        row = model[path]
-        if row[TAG] in Massager.fmt:
-            fmt = Massager.fmt[row[TAG]]
-            newnew = fmt.validate(new)
-            if not newnew:
-                qltk.WarningMessage(
-                    None, _("Invalid value"), _("Invalid value") +
-                    (": <b>%s</b>\n\n%s" % (new, fmt.error))).run()
-                return
-            else: new = newnew
-        if row[VALUE].replace('<i>','').replace('</i>','') != new:
-            row[VALUE] = util.escape(new)
-            row[EDITED] = True
-            row[DELETED] = False
+            if row[ORIGVALUE] is None:
+                # The tag hasn't been saved yet, so we can just update
+                # the name in the model, and the value, since it
+                # may have been re-validated.
+                row[TAG] = new_tag
+                row[VALUE] = value
+            else:
+                # The tag has been saved, so delete the old tag and
+                # add a new one with the old (or sanitized) value.
+                row[RENAMED] = row[EDITED] = True
+                row[ORIGTAG] = row[TAG]
+                row[TAG] = new_tag
 
     def __button_press(self, view, event):
         if event.button not in [1, 2]: return False
@@ -571,42 +738,63 @@ class EditTags(gtk.VBox):
         else: return False
 
     def __update(self, songs, view, buttonbox, model, add, buttons):
-        if songs is None: songs = self.__songs
+        if songs is None:
+            songs = self.__songinfo.songs
 
-        from library import AudioFileGroup
         self.__songinfo = songinfo = AudioFileGroup(songs)
-        self.__songs = songs
         view.set_model(None)
         model.clear()
         view.set_model(model)
 
-        keys = songinfo.realkeys()
-        keys.sort()
+        keys = sorted(songinfo.realkeys())
 
         if not config.getboolean("editing", "alltags"):
-            keys = filter(lambda k: k not in const.MACHINE_TAGS, keys)
+            keys = filter(lambda k: k not in formats.MACHINE_TAGS, keys)
 
         # reverse order here so insertion puts them in proper order.
-        for comment in ['album', 'artist', 'title']:
-            try: keys.remove(comment)
+        for tag in ['album', 'artist', 'title']:
+            try: keys.remove(tag)
             except ValueError: pass
-            else: keys.insert(0, comment)
+            else: keys.insert(0, tag)
 
-        for comment in keys:
+        for tag in keys:
             # Handle with care.
-            orig_value = songinfo[comment].split("\n")
-            value = songinfo[comment].safenicestr()
+            orig_value = songinfo[tag].split("\n")
+            value = songinfo[tag].safenicestr()
             edited = False
-            edit = songinfo.can_change(comment)
+            edit = songinfo.can_change(tag)
             deleted = False
+            renamed = False
+            newtag = ""
             if value[0:1] == "<": # "different etc."
-                model.append(row=[comment, value, edited, edit, deleted,
-                                  "\n".join(orig_value)])
+                model.append(row=[tag, value, edited, edit, deleted,
+                                  "\n".join(orig_value), renamed,
+                                  newtag])
             else:
                 for i, v in enumerate(value.split("\n")):
-                    model.append(row=[comment, v, edited, edit, deleted,
-                                      orig_value[i]])
+                    model.append(row=[tag, v, edited, edit, deleted,
+                                      orig_value[i], renamed, newtag])
 
         buttonbox.set_sensitive(bool(songinfo.can_change()))
         for b in buttons: b.set_sensitive(False)
         add.set_sensitive(bool(songs))
+
+    def __value_editing_started(self, render, editable, path, model, library):
+        try:
+            if not editable.get_completion():
+                tag = model[path][TAG]
+                completion = LibraryValueCompletion(tag, library)
+                editable.set_completion(completion)
+        except AttributeError:
+            pass
+
+    def __tag_editing_started(self, render, editable, path, model, library):
+        try:
+            if not editable.get_completion():
+                tags = self.__songinfo.can_change()
+                if tags == True:
+                    from formats import USEFUL_TAGS as tags
+                completion = qltk.EntryCompletion(tags)
+                editable.set_completion(completion)
+        except AttributeError:
+            pass
