@@ -1,4 +1,3 @@
-
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
@@ -6,21 +5,22 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: information.py 2621 2005-12-31 03:02:08Z piman $
+# $Id: information.py 3634 2006-07-14 19:38:41Z piman $
 
-import os, sys
 import time
-import locale
-import gtk, pango
+
+import gtk
+import pango
+
+import const
 import qltk
+import util
+
+from qltk.bookmarks import EditBookmarksPane
 from qltk.cover import CoverImage
-from qltk.x import Window
 from qltk.lyrics import LyricsPane
-
-import util; from util import tag
-
-if sys.version_info < (2, 4): from sets import Set as set
-import __builtin__; __builtin__.__dict__.setdefault("_", lambda a: a)
+from qltk.x import Window
+from util import tag
 
 def Label(*args):
     l = gtk.Label(*args)
@@ -51,7 +51,7 @@ class NoSongs(gtk.Label):
         self.title = _("No Songs")
 
 class OneSong(qltk.Notebook):
-    def __init__(self, song):
+    def __init__(self, library, song):
         super(OneSong, self).__init__()
         vbox = gtk.VBox(spacing=12)
         vbox.set_border_width(12)
@@ -68,6 +68,26 @@ class OneSong(qltk.Notebook):
         lyrics.title = _("Lyrics")
         self.append_page(lyrics)
 
+        bookmarks = EditBookmarksPane(None, song)
+        bookmarks.title = _("Bookmarks")
+        bookmarks.set_border_width(12)
+        self.append_page(bookmarks)
+
+        s = library.connect('changed', self.__check_changed, vbox, song)
+        self.connect_object('destroy', library.disconnect, s)
+
+    def __check_changed(self, library, songs, vbox, song):
+        if song in songs:
+            for c in vbox.get_children():
+                vbox.remove(c)
+                c.destroy()
+            self._title(song, vbox)
+            self._album(song, vbox)
+            self._people(song, vbox)
+            self._library(song, vbox)
+            self._file(song, vbox)
+            vbox.show_all()
+
     def _title(self, song, box):
         l = Label()
         text = "<big><b>%s</b></big>" % util.escape(song("title"))
@@ -146,9 +166,11 @@ class OneSong(qltk.Notebook):
             if i == 0: return _("Never")
             else: return ngettext("%d time", "%d times", i) % i
         def ftime(t):
-            if t == 0: return _("Unknown")
-            else: return time.strftime("%c", time.localtime(t)).decode(
-                locale.getpreferredencoding())
+            if t == 0:
+                return _("Unknown")
+            else:
+                timestr = time.strftime("%c", time.localtime(t))
+                return timestr.decode(const.ENCODING)
 
         playcount = counter(song.get("~#playcount", 0))
         skipcount = counter(song.get("~#skipcount", 0))
@@ -178,9 +200,11 @@ class OneSong(qltk.Notebook):
 
     def _file(self, song, box):
         def ftime(t):
-            if t == 0: return _("Unknown")
-            else: return time.strftime("%c", time.localtime(t)).decode(
-                locale.getpreferredencoding())
+            if t == 0:
+                return _("Unknown")
+            else:
+                timestr = time.strftime("%c", time.localtime(t))
+                return timestr.decode(const.ENCODING)
 
         fn = util.fsdecode(util.unexpand(song["~filename"]))
         length = util.format_time_long(song["~#length"])
@@ -218,9 +242,8 @@ class OneAlbum(qltk.Notebook):
         vbox = gtk.VBox(spacing=12)
         vbox.set_border_width(12)
         swin.add_with_viewport(vbox)
-        songs = list(songs)
         # Needed to get proper track/disc/part ordering
-        songs.sort()
+        songs = sorted(songs)
         self._title(songs, vbox)
         self._album(songs, vbox)
         self._people(songs, vbox)
@@ -296,8 +319,8 @@ class OneAlbum(qltk.Notebook):
             artists.update(song.list("artist"))
             performers.update(song.list("performer"))
 
-        artists = list(artists); artists.sort()
-        performers = list(performers); performers.sort()
+        artists = sorted(artists)
+        performers = sorted(performers)
 
         if artists:
             if len(artists) == 1: title = _("artist")
@@ -432,8 +455,7 @@ class ManySongs(qltk.Notebook):
         for song in songs:
             if "artist" in song: artists.update(song.list("artist"))
             else: none += 1
-        artists = list(artists)
-        artists.sort()
+        artists = sorted(artists)
         num_artists = len(artists)
 
         if none: artists.append(ngettext("%d song with no artist",
@@ -449,8 +471,7 @@ class ManySongs(qltk.Notebook):
         for song in songs:
             if "album" in song: albums.update(song.list("album"))
             else: none += 1
-        albums = list(albums)
-        albums.sort()
+        albums = sorted(albums)
         num_albums = len(albums)
 
         if none: albums.append(ngettext("%d song with no album",
@@ -480,37 +501,39 @@ class ManySongs(qltk.Notebook):
                        expand=False, fill=False)
 
 class Information(Window):
-    def __init__(self, watcher, songs):
+    def __init__(self, library, songs):
         super(Information, self).__init__()
         self.set_border_width(12)
         self.set_default_size(400, 400)
-        s1 = watcher.connect('changed', self.__check_changed)
-        s2 = watcher.connect('removed', self.__check_removed)
-        self.connect_object('destroy', watcher.disconnect, s1)
-        self.connect_object('destroy', watcher.disconnect, s2)
+        if len(songs) > 1:
+            sig = library.connect('changed', self.__check_changed)
+            self.connect_object('destroy', library.disconnect, sig)
+        if len(songs) > 0:
+            sig = library.connect('removed', self.__check_removed)
+            self.connect_object('destroy', library.disconnect, sig)
         self.__songs = songs
-        self.__update()
+        self.__update(library)
         self.show_all()
 
-    def __check_changed(self, watcher, songs):
+    def __check_changed(self, library, songs):
         changed = set(songs)
         for song in self.__songs:
             if song in changed:
-                self.__update()
+                self.__update(library)
                 break
 
-    def __check_removed(self, watcher, songs):
+    def __check_removed(self, library, songs):
         gone = set(songs)
         old = len(self.__songs)
         self.__songs = filter(lambda s: s not in gone, self.__songs)
-        if len(self.__songs) != old: self.__update()
+        if len(self.__songs) != old: self.__update(library)
 
-    def __update(self):
+    def __update(self, library):
         songs = self.__songs
         if self.child: self.child.destroy()
         self.__songs = songs
         if not songs: self.add(NoSongs())
-        elif len(songs) == 1: self.add(OneSong(songs[0]))
+        elif len(songs) == 1: self.add(OneSong(library, songs[0]))
         else:
             tags = [(s.get("artist"), s.get("album")) for s in songs]
             artists, albums = zip(*tags)
