@@ -5,35 +5,46 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: quodlibet.py 2909 2006-03-01 18:53:49Z piman $
+# $Id: quodlibet.py 3807 2006-08-28 05:39:03Z piman $
 
-import os, sys
+import os
+import random
+import sys
 
-import gtk, pango, gobject, gst
-import stock
-import qltk
+import gst
+import gtk
+import pango
 
 import browsers
-import const
 import config
-import player
+import const
 import formats
+import player
+import qltk
+import qltk.about
+import stock
 import util
-import locale
-import widgets
 
-from library import library
-
-if sys.version_info < (2, 4): from sets import Set as set
-from qltk.properties import SongProperties
-from qltk.songlist import SongList
-from qltk.wlw import WaitLoadWindow
-from qltk.getstring import GetStringDialog
-from qltk.browser import LibraryBrowser
-from qltk.msg import ErrorMessage
-from qltk.information import Information
-from util.uri import URI
+from formats.remote import RemoteFile
+from library import library, librarian
 from parse import Query
+from qltk.browser import LibraryBrowser
+from qltk.chooser import FolderChooser, FileChooser
+from qltk.controls import PlayControls
+from qltk.cover import CoverImage
+from qltk.getstring import GetStringDialog
+from qltk.info import SongInfo
+from qltk.information import Information
+from qltk.mmkeys import MmKeys
+from qltk.msg import ErrorMessage
+from qltk.playorder import PlayOrder
+from qltk.pluginwin import PluginWindow
+from qltk.properties import SongProperties
+from qltk.prefs import PreferencesWindow
+from qltk.queue import QueueExpander
+from qltk.songlist import SongList, PlaylistMux
+from util import copool
+from util.uri import URI
 
 class MainSongList(SongList):
     # The SongList that represents the current playlist.
@@ -49,22 +60,24 @@ class MainSongList(SongList):
                  pixbuf=(gtk.STOCK_MEDIA_PLAY, gtk.STOCK_MEDIA_PAUSE)):
             try:
                 if model.get_path(iter) != model.current_path: stock = ''
-                else: stock = pixbuf[player.playlist.paused]
+                elif model.sourced:
+                    stock = pixbuf[player.playlist.paused]
+                else: stock = gtk.STOCK_MEDIA_STOP
                 cell.set_property('stock-id', stock)
             except AttributeError: pass
 
         def __init__(self):
-            gtk.TreeViewColumn.__init__(self, "", self._render)
+            super(MainSongList.CurrentColumn, self).__init__("", self._render)
             self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             self.set_fixed_width(24)
             self.set_cell_data_func(self._render, self._cdf)
             self.header_name = "~current"
 
-    def __init__(self, watcher, player, visible):
-        super(MainSongList, self).__init__(watcher)
+    def __init__(self, library, player, visible):
+        super(MainSongList, self).__init__(library, player)
         self.set_rules_hint(True)
-        s = watcher.connect_object('removed', map, player.remove)
-        self.connect_object('destroy', watcher.disconnect, s)
+        s = library.librarian.connect_object('removed', map, player.remove)
+        self.connect_object('destroy', library.librarian.disconnect, s)
         self.connect_object('row-activated', self.__select_song, player)
         self.connect_object('notify::visible', self.__visibility, visible)
 
@@ -77,13 +90,56 @@ class MainSongList(SongList):
         if player.song: player.paused = False
 
     def set_sort_by(self, *args, **kwargs):
-        SongList.set_sort_by(self, *args, **kwargs)
+        super(MainSongList, self).set_sort_by(*args, **kwargs)
         tag, reverse = self.get_sort_by()
         config.set('memory', 'sortby', "%d%s" % (int(reverse), tag))
 
+class StatusBar(gtk.HBox):
+    def __init__(self):
+        super(StatusBar, self).__init__()
+        self.progress = gtk.ProgressBar()
+        self.count = gtk.Label(_("No time information"))
+        self.count.set_justify(gtk.JUSTIFY_RIGHT)
+        self.count.set_ellipsize(pango.ELLIPSIZE_START)
+        self.pack_start(self.count)
+        progress_label = gtk.Label()
+        progress_label.set_justify(gtk.JUSTIFY_RIGHT)
+        progress_label.set_ellipsize(pango.ELLIPSIZE_START)
+        # GtkProgressBar can't show text when pulsing. Proxy its set_text
+        # method to a label that can.
+        self.progress.set_text = progress_label.set_text
+        hb = gtk.HBox(spacing=12)
+        hb.pack_start(progress_label)
+        hb.pack_start(self.progress, expand=False)
+        pause = gtk.ToggleButton()
+        pause.add(gtk.image_new_from_stock(
+            gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
+        pause.connect('toggled', self.__pause)
+        hb.pack_start(pause, expand=False)
+        self.pack_start(hb)
+        self.progress.connect('notify::visible', self.__toggle, pause, hb)
+        self.progress.connect_object(
+            'notify::fraction', lambda *args: args[0].set_active(False), pause)
+        self.count.show()
+
+    def __pause(self, pause):
+        if pause.get_active():
+            copool.pause("library")
+        else:
+            copool.resume("library")
+
+    def __toggle(self, bar, property, pause, hb):
+        if self.progress.props.visible:
+            self.count.hide()
+            pause.set_active(False)
+            hb.show_all()
+        else:
+            self.count.show()
+            hb.hide()
+
 class QuodLibetWindow(gtk.Window):
-    def __init__(self, watcher, player):
-        gtk.Window.__init__(self)
+    def __init__(self, library, player):
+        super(QuodLibetWindow, self).__init__()
         self.last_dir = os.path.expanduser("~")
 
         tips = qltk.Tooltips(self)
@@ -97,7 +153,7 @@ class QuodLibetWindow(gtk.Window):
         self.__create_menu(tips, player)
         self.add_accel_group(self.ui.get_accel_group())
 
-        accel_fn = os.path.join(const.DIR, "accels")
+        accel_fn = os.path.join(const.USERDIR, "accels")
         gtk.accel_map_load(accel_fn)
         accelgroup = gtk.accel_groups_from_object(self)[0]
         accelgroup.connect('accel-changed',
@@ -109,36 +165,31 @@ class QuodLibetWindow(gtk.Window):
         self.child.pack_start(realvbox)
 
         # get the playlist up before other stuff
-        from qltk.queue import QueueExpander
         self.songlist = MainSongList(
-            watcher, player, self.ui.get_widget("/Menu/View/SongList"))
+            library, player, self.ui.get_widget("/Menu/View/SongList"))
         self.add_accel_group(self.songlist.accelerators)
         self.songlist.connect_after(
             'drag-data-received', self.__songlist_drag_data_recv)
         self.qexpander = QueueExpander(
-            self.ui.get_widget("/Menu/View/Queue"), watcher)
-        from qltk.songlist import PlaylistMux
+            self.ui.get_widget("/Menu/View/Queue"), library, player)
         self.playlist = PlaylistMux(
-            watcher, self.qexpander.model, self.songlist.model)
+            player, self.qexpander.model, self.songlist.model)
 
         # song info (top part of window)
         hbox = gtk.HBox(spacing=6)
 
         # play controls
-        from qltk.controls import PlayControls
-        t = PlayControls(watcher, player)
+        t = PlayControls(player, library.librarian)
         self.volume = t.volume
         hbox.pack_start(t, expand=False, fill=False)
 
         # song text
-        from qltk.info import SongInfo
-        text = SongInfo(watcher, player)
+        text = SongInfo(library.librarian, player)
         hbox.pack_start(text)
 
         # cover image
-        from qltk.cover import CoverImage
         self.image = CoverImage()
-        watcher.connect('song-started', self.image.set_song)
+        player.connect('song-started', self.image.set_song)
         hbox.pack_start(self.image, expand=False)
 
         realvbox.pack_start(hbox, expand=False)
@@ -148,9 +199,9 @@ class QuodLibetWindow(gtk.Window):
         align.set_padding(0, 6, 6, 6)
         hbox = gtk.HBox(spacing=12)
         hb = gtk.HBox(spacing=3)
-        from qltk.playorder import PlayOrder
         label = gtk.Label(_("_Order:"))
-        self.order = order = PlayOrder(self.songlist.model)
+        label.set_size_request(-1, 28)
+        self.order = order = PlayOrder(self.songlist.model, player)
         label.set_mnemonic_widget(order)
         label.set_use_underline(True)
         hb.pack_start(label)
@@ -160,18 +211,14 @@ class QuodLibetWindow(gtk.Window):
             _("_Repeat"), "settings", "repeat")
         tips.set_tip(repeat, _("Restart the playlist when finished"))
         hbox.pack_start(repeat, expand=False)
-        self.__statusbar = gtk.Label()
-        self.__statusbar.set_text(_("No time information"))
-        # GTK (2.8?) rounding error, 1.0 clips the rightmost pixel.
-        self.__statusbar.set_alignment(0.999999, 0.5)
-        self.__statusbar.set_ellipsize(pango.ELLIPSIZE_START)
+        self.__statusbar = StatusBar()
         hbox.pack_start(self.__statusbar)
         align.add(hbox)
         self.child.pack_end(align, expand=False)
 
         # song list
         self.song_scroller = sw = gtk.ScrolledWindow()
-        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
         sw.set_shadow_type(gtk.SHADOW_IN)
         sw.add(self.songlist)
 
@@ -189,12 +236,12 @@ class QuodLibetWindow(gtk.Window):
 
         self.browser = None
 
-        from qltk.mmkeys import MmKeys
         self.__keys = MmKeys(player)
 
         self.child.show_all()
         sw.show_all()
-        self.select_browser(self, config.get("memory", "browser"), player)
+        self.select_browser(
+            self, config.get("memory", "browser"), library, player)
         self.browser.restore()
         self.browser.activate()
         self.showhide_playlist(self.ui.get_widget("/Menu/View/SongList"))
@@ -212,13 +259,13 @@ class QuodLibetWindow(gtk.Window):
         self.songlist.connect('columns-changed', self.__hide_headers)
         self.songlist.get_selection().connect('changed', self.__set_time)
 
-        watcher.connect('removed', self.__set_time)
-        watcher.connect('refresh', self.__refresh)
-        watcher.connect('changed', self.__update_title, player)
-        watcher.connect('song-started', self.__song_started, player)
-        watcher.connect_after('song-ended', self.__song_ended, player)
-        watcher.connect('paused', self.__update_paused, True)
-        watcher.connect('unpaused', self.__update_paused, False)
+        library.librarian.connect('removed', self.__set_time)
+        library.librarian.connect('added', self.__set_time)
+        library.librarian.connect_object('changed', self.__update_title, player)
+        player.connect('song-ended', self.__song_ended)
+        player.connect('song-started', self.__song_started)
+        player.connect('paused', self.__update_paused, True)
+        player.connect('unpaused', self.__update_paused, False)
 
         targets = [("text/uri-list", 0, 1)]
         self.drag_dest_set(
@@ -229,6 +276,7 @@ class QuodLibetWindow(gtk.Window):
             'drag-data-received', QuodLibetWindow.__drag_data_received, self)
 
         self.resize(*map(int, config.get("memory", "size").split()))
+        self.__rebuild(None, False)
 
     def __drag_motion(self, ctx, x, y, time):
         # Don't accept drops from QL itself, since it offers text/uri-list.
@@ -249,7 +297,6 @@ class QuodLibetWindow(gtk.Window):
         dirs = []
         files = []
         error = False
-        from formats.remote import RemoteFile
         for uri in uris:
             try: uri = URI(uri)
             except ValueError: continue
@@ -260,12 +307,12 @@ class QuodLibetWindow(gtk.Window):
                 else:
                     loc = os.path.realpath(loc)
                     if loc not in library:
-                        song = library.add(loc)
+                        song = library.add_filename(loc)
                         if song: files.append(song)
             elif gst.element_make_from_uri(gst.URI_SRC, uri, ''):
                 if uri not in library:
                     files.append(RemoteFile(uri))
-                    library.add_song(files[-1])
+                    library.add([files[-1]])
             else:
                 error = True
                 break
@@ -275,8 +322,9 @@ class QuodLibetWindow(gtk.Window):
                 self, _("Unable to add songs"),
                 _("<b>%s</b> uses an unsupported protocol.") % uri).run()
         else:
-            if dirs: self.scan_dirs(dirs)
-            if files: widgets.watcher.added(files)
+            if dirs:
+                copool.add(library.scan, dirs, self.__status.bar.progress,
+                           funcid="library")
 
     def __songlist_drag_data_recv(self, view, *args):
         if callable(self.browser.reordered): self.browser.reordered(view)
@@ -347,7 +395,6 @@ class QuodLibetWindow(gtk.Window):
              self.__current_song_prop),
             ("Information", gtk.STOCK_INFO, None, None, None,
              self.__current_song_info),
-            ("Rating", None, _("_Rating")),
 
             ("Jump", gtk.STOCK_JUMP_TO, _("_Jump to Playing Song"),
              "<control>J", None, self.__jump_to_current),
@@ -367,19 +414,18 @@ class QuodLibetWindow(gtk.Window):
 
         ag.add_actions(actions)
 
-        from qltk.about import AboutWindow
         act = gtk.Action("About", None, None, gtk.STOCK_ABOUT)
-        act.connect_object('activate', AboutWindow, self, player)
-        ag.add_action(act)
+        act.connect_object('activate', qltk.about.show, self, player)
+        ag.add_action_with_accel(act, None)
 
         act = gtk.Action(
             "RefreshLibrary", _("Re_fresh Library"), None, gtk.STOCK_REFRESH)
-        act.connect('activate', self.__rebuild)
-        ag.add_action(act)
+        act.connect('activate', self.__rebuild, False)
+        ag.add_action_with_accel(act, None)
         act = gtk.Action(
             "ReloadLibrary", _("Re_load Library"), None, gtk.STOCK_REFRESH)
         act.connect('activate', self.__rebuild, True)
-        ag.add_action(act)
+        ag.add_action_with_accel(act, None)
 
         for tag_, lab in [
             ("genre", _("Filter on _Genre")),
@@ -388,7 +434,7 @@ class QuodLibetWindow(gtk.Window):
             act = gtk.Action(
                 "Filter%s" % util.capitalize(tag_), lab, None, gtk.STOCK_INDEX)
             act.connect_object('activate', self.__filter_on, tag_, None, player)
-            ag.add_action(act)
+            ag.add_action_with_accel(act, None)
 
         for (tag_, accel, label) in [
             ("genre", "G", _("Random _Genre")),
@@ -409,17 +455,22 @@ class QuodLibetWindow(gtk.Window):
              self.showhide_playqueue,
              config.getboolean("memory", "queue"))])
 
-        ag.add_radio_actions([
-            (a, None, l, None, None, i) for (i, (a, l, K)) in
-            enumerate(browsers.get_view_browsers())
-            ], browsers.index(config.get("memory", "browser")),
-                             self.select_browser, player)
+        view_actions = []
+        for i, Kind in enumerate(browsers.browsers):
+            action = "View" + Kind.__name__
+            label = Kind.accelerated_name
+            view_actions.append((action, None, label, None, None, i))
+        current = browsers.index(config.get("memory", "browser"))
+        ag.add_radio_actions(
+            view_actions, current, self.select_browser, (library, player))
 
-        for id, label, Kind in browsers.get_browsers():
-            act = gtk.Action(id, label, None, None)
-            act.connect_object(
-                'activate', LibraryBrowser, Kind, widgets.watcher)
-            ag.add_action(act)
+        for Kind in browsers.browsers:
+            if not Kind.in_menu: continue
+            action = "Browser" + Kind.__name__
+            label = Kind.accelerated_name
+            act = gtk.Action(action, label, None, None)
+            act.connect_object('activate', LibraryBrowser, Kind, library)
+            ag.add_action_with_accel(act, None)
 
         self.ui = gtk.UIManager()
         self.ui.insert_action_group(ag, -1)
@@ -449,7 +500,7 @@ class QuodLibetWindow(gtk.Window):
             key = "%s_pos" % browser.__class__.__name__
             config.set("browsers", key, str(paned.get_relative()))
 
-    def select_browser(self, activator, current, player):
+    def select_browser(self, activator, current, library, player):
         if isinstance(current, gtk.RadioAction):
             current = current.get_current_value()
         Browser = browsers.get(current)
@@ -462,9 +513,12 @@ class QuodLibetWindow(gtk.Window):
             if self.browser.accelerators:
                 self.remove_accel_group(self.browser.accelerators)
             self.browser.destroy()
-        self.browser = Browser(widgets.watcher, player)
+        self.browser = Browser(library, player)
         self.browser.connect('songs-selected', self.__browser_cb)
-        if self.browser.reordered: self.songlist.enable_drop()
+        if self.browser.reordered:
+            self.songlist.enable_drop()
+        elif self.browser.dropped:
+            self.songlist.enable_drop(False)
         else: self.songlist.disable_drop()
         if self.browser.accelerators:
             self.add_accel_group(self.browser.accelerators)
@@ -490,13 +544,15 @@ class QuodLibetWindow(gtk.Window):
             c = gtk.VBox(spacing=6)
             c.pack_start(self.browser, expand=False)
             c.pack_start(self.songpane)
+        player.replaygain_profiles[0] = self.browser.replaygain_profiles
+        player.volume = player.volume
         self.__vbox.pack_end(c)
         c.show()
         self.__hide_menus()
         self.__hide_headers()
         self.__refresh_size()
 
-    def __update_paused(self, watcher, paused):
+    def __update_paused(self, player, paused):
         menu = self.ui.get_widget("/Menu/Control/PlayPause")
         if paused: key = gtk.STOCK_MEDIA_PLAY
         else: key = gtk.STOCK_MEDIA_PAUSE
@@ -505,7 +561,7 @@ class QuodLibetWindow(gtk.Window):
         menu.child.set_text(text)
         menu.child.set_use_underline(True)
 
-    def __song_ended(self, watcher, song, stopped, player):
+    def __song_ended(self, player, song, stopped):
         if song is None: return
         if not self.browser.dynamic(song):
             player.remove(song)
@@ -514,15 +570,16 @@ class QuodLibetWindow(gtk.Window):
                 self.songlist.model.remove(iter)
                 self.__set_time()
 
-    def __update_title(self, watcher, songs, player):
+    def __update_title(self, player, songs):
         if player.song in songs:
-            song = player.song
+            song = player.info
             if song:
                 self.set_title("Quod Libet - " + song.comma("~title~version"))
             else: self.set_title("Quod Libet")
 
-    def __song_started(self, watcher, song, player):
-        self.__update_title(watcher, [song], player)
+    def __song_started(self, player, song):
+        if song is None:
+            self.__update_title(player, [song])
 
         for wid in ["Jump", "Next", "EditTags", "Information"]:
             self.ui.get_widget('/Menu/Control/'+wid).set_sensitive(bool(song))
@@ -561,7 +618,6 @@ class QuodLibetWindow(gtk.Window):
     def __play_pause(self, *args):
         if player.playlist.song is None:
             player.playlist.reset()
-            player.playlist.next()
         else: player.playlist.paused ^= True
 
     def __jump_to_current(self, explicit):
@@ -570,6 +626,11 @@ class QuodLibetWindow(gtk.Window):
             path = self.songlist.model.current_path
             self.songlist.scroll_to_cell(
                 path[0], use_align=True, row_align=0.5)
+            if explicit:
+                iter = self.songlist.model.current_iter
+                selection = self.songlist.get_selection()
+                selection.unselect_all()
+                selection.select_path(path)
         if explicit: self.browser.scroll(player.playlist.song)
 
     def __next_song(self, *args): player.playlist.next()
@@ -580,8 +641,10 @@ class QuodLibetWindow(gtk.Window):
 
     def __random(self, item, key):
         if self.browser.can_filter(key):
-            value = library.random(key)
-            if value is not None: self.browser.filter(key, [value])
+            values = self.browser.list(key)
+            if values:
+                value = random.choice(values)
+                self.browser.filter(key, [value])
 
     def lastplayed_day(self, menuitem):
         self.__make_query("#(lastplayed > today)")
@@ -593,7 +656,7 @@ class QuodLibetWindow(gtk.Window):
         self.__make_query("#(playcount = 0)")
 
     def __top40(self, menuitem):
-        songs = [song["~#playcount"] for song in library.itervalues()]
+        songs = [song["~#playcount"] for song in library]
         if len(songs) == 0: return
         songs.sort()
         if len(songs) < 40:
@@ -602,7 +665,7 @@ class QuodLibetWindow(gtk.Window):
             self.__make_query("#(playcount > %d)" % (songs[-40] - 1))
 
     def __bottom40(self, menuitem):
-        songs = [song["~#playcount"] for song in library.itervalues()]
+        songs = [song["~#playcount"] for song in library]
         if len(songs) == 0: return
         songs.sort()
         if len(songs) < 40:
@@ -610,43 +673,17 @@ class QuodLibetWindow(gtk.Window):
         else:
             self.__make_query("#(playcount < %d)" % (songs[-40] + 1))
 
-    def __rebuild(self, activator, hard=False):
-        self.__keys.block()
-        window = WaitLoadWindow(self, len(library) // 7,
-                                _("Scanning your library. "
-                                  "This may take several minutes.\n\n"
-                                  "%d songs reloaded\n%d songs removed"),
-                                (0, 0))
-        iter = 7
-        c = []
-        r = []
-        s = False
-        for c, r in library.rebuild(hard):
-            if iter == 7:
-                if window.step(len(c), len(r)):
-                    window.destroy()
-                    break
-                iter = 0
-            iter += 1
-        else:
-            window.destroy()
-            if config.get("settings", "scan"):
-                s = self.scan_dirs(config.get("settings", "scan").split(":"))
-        widgets.watcher.changed(c)
-        widgets.watcher.removed(r)
-        if c or r or s:
-            library.save(const.LIBRARY)
-            widgets.watcher.refresh()
-        self.__keys.unblock()
+    def __rebuild(self, activator, force):
+        paths = config.get("settings", "scan").split(":")
+        copool.add(library.rebuild, paths, self.__statusbar.progress, force,
+                   funcid="library")
 
     # Set up the preferences window.
     def __preferences(self, activator):
-        from qltk.prefs import PreferencesWindow
         PreferencesWindow(self)
 
     def __plugins(self, activator):
-        from qltk.pluginwin import PluginWindow
-        PluginWindow(self, SongList.pm)
+        PluginWindow(self)
 
     def open_location(self, action):
         name = GetStringDialog(self, _("Add a Location"),
@@ -664,38 +701,37 @@ class QuodLibetWindow(gtk.Window):
                     _("<b>%s</b> uses an unsupported protocol.") %(
                     util.escape(name))).run()
             else:
-                from formats.remote import RemoteFile
                 if name not in library:
-                    song = library.add_song(RemoteFile(name))
-                    if song: widgets.watcher.added([song])
+                    song = library.add([RemoteFile(name)])
 
     def open_chooser(self, action):
         if not os.path.exists(self.last_dir):
-            self.last_dir = os.environ["HOME"]
+            self.last_dir = const.HOME
 
         if action.get_name() == "AddFolders":
-            from qltk.chooser import FolderChooser
             chooser = FolderChooser(self, _("Add Music"), self.last_dir)
+            cb = gtk.CheckButton(_("Watch this folder for new songs"))
+            cb.set_active(not config.get("settings", "scan"))
+            cb.show()
+            chooser.set_extra_widget(cb)
         else:
-            from qltk.chooser import FileChooser
             chooser = FileChooser(
                 self, _("Add Music"), formats.filter, self.last_dir)
+            cb = None
         
         fns = chooser.run()
         chooser.destroy()
         if fns:
             if action.get_name() == "AddFolders":
                 self.last_dir = fns[0]
-                if self.scan_dirs(fns):
-                    widgets.watcher.refresh()
-                    self.browser.activate()
-                    library.save(const.LIBRARY)
+                copool.add(library.scan, fns, self.__statusbar.progress,
+                           funcid="library")
             else:
                 added = []
                 self.last_dir = os.path.basename(fns[0])
                 for filename in map(os.path.realpath, fns):
                     if filename in library: continue
-                    song = library.add(filename)
+                    song = library.add_filename(filename)
                     if song: added.append(song)
                     else:
                         from traceback import format_exception_only as feo
@@ -703,44 +739,37 @@ class QuodLibetWindow(gtk.Window):
                         msg = _("%s could not be added to your library.\n\n")
                         msg %= util.escape(util.fsdecode(
                             os.path.basename(filename)))
-                        msg += util.escape(util.fsdecode(
-                            "".join(tb).decode(locale.getpreferredencoding())))
+                        msg += util.escape("".join(tb).decode(
+                            const.ENCODING, "replace"))
                         d = ErrorMessage(self, _("Unable to add song"), msg)
                         d.label.set_selectable(True)
                         d.run()
                         continue
                 if added:
-                    widgets.watcher.added(added)
                     self.browser.activate()
 
-    def scan_dirs(self, fns):
-        win = WaitLoadWindow(self, 0,
-                             _("Scanning for new songs and "
-                               "adding them to your library.\n\n"
-                               "%d songs added"), 0)
-        added, changed, removed = [], [], []
-        for added, changed, removed in library.scan(fns):
-            if win.step(len(added)): break
-        widgets.watcher.changed(changed)
-        widgets.watcher.added(added)
-        widgets.watcher.removed(removed)
-        win.destroy()
-        return (added or changed or removed)
+        if cb and cb.get_active():
+            dirs = config.get("settings", "scan").split(":")
+            for fn in fns:
+                if fn not in dirs: dirs.append(fn)
+            dirs = ":".join(dirs)
+            config.set("settings", "scan", dirs)
 
     def __songs_popup_menu(self, songlist):
         path, col = songlist.get_cursor()
         header = col.header_name
-        menu = self.songlist.Menu(header, self.browser, widgets.watcher)
-        menu.popup(None, None, None, 0, gtk.get_current_event_time())
-        return True
+        menu = self.songlist.Menu(header, self.browser, library)
+        if menu is not None:
+            return self.songlist.popup_menu(menu, 0,
+                    gtk.get_current_event_time())
 
     def __current_song_prop(self, *args):
         song = player.playlist.song
-        if song: SongProperties(widgets.watcher, [song])
+        if song: SongProperties(librarian, [song])
 
     def __current_song_info(self, *args):
         song = player.playlist.song
-        if song: Information(widgets.watcher, [song])
+        if song: Information(librarian, [song])
 
     def __hide_menus(self):
         menus = {'genre': ["/Menu/Filters/FilterGenre",
@@ -811,18 +840,12 @@ class QuodLibetWindow(gtk.Window):
             self.browser.set_text(query.encode('utf-8'))
             self.browser.activate()
 
-    def __refresh(self, watcher):
-        self.__set_time()
-
     def __set_time(self, *args, **kwargs):
-        statusbar = self.__statusbar
         songs = kwargs.get("songs") or self.songlist.get_selected_songs()
         if "songs" not in kwargs and len(songs) <= 1:
             songs = self.songlist.get_songs()
-
         i = len(songs)
         length = sum([song["~#length"] for song in songs])
         t = self.browser.statusbar(i) % {
             'count': i, 'time': util.format_time_long(length)}
-        statusbar.set_text(t)
-        gobject.idle_add(statusbar.queue_resize)
+        self.__statusbar.count.set_text(t)
