@@ -4,42 +4,44 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: songlist.py 2910 2006-03-01 19:30:34Z piman $
+# $Id: songlist.py 3722 2006-07-30 18:15:14Z piman $
 
-import os, sys
+import datetime
 import random
-import locale
-import time, datetime
+import time
 
-import gobject, gtk, pango
+import gobject
+import gtk
+import pango
 
-import stock
-import qltk
+import config
+import const
 import player
-import util; from util import tag
-from library import library
-from qltk.properties import SongProperties
-from qltk.information import Information
-from qltk.views import AllTreeView
-from qltk.delete import DeleteDialog
-from util.uri import URI
-from parse import Query
+import qltk
+import util
 
-if sys.version_info < (2, 4): from sets import Set as set
+from parse import Query, Pattern
+from qltk.information import Information
+from qltk.properties import SongProperties
+from qltk.views import AllTreeView
+from util import tag
+from util import copool
+from util.uri import URI
 
 OFF, SHUFFLE, WEIGHTED, ONESONG = range(4)
 
 class PlaylistMux(object):
-    def __init__(self, watcher, q, pl):
+
+    def __init__(self, player, q, pl):
         self.q = q
         self.pl = pl
-        watcher.connect('song-started', self.__check_q)
+        player.connect('song-started', self.__check_q)
 
-    def __check_q(self, watcher, song):
+    def __check_q(self, player, song):
         if song is not None:
             iter = self.q.find(song)
             if iter: self.q.remove(iter)
-            self.q.go_to(None)
+            self.q.reset()
 
     def get_current(self):
         if self.q.current is not None: return self.q.current
@@ -48,12 +50,24 @@ class PlaylistMux(object):
     current = property(get_current)
 
     def next(self):
-        if self.q.is_empty(): self.pl.next()
-        elif self.q.current is None: self.q.next()
+        if self.q.is_empty():
+            self.pl.next()
+            self.q.sourced = False
+            self.pl.sourced = True
+        elif self.q.current is None:
+            self.q.next()
+            self.q.sourced = True
+            self.pl.sourced = False
 
     def next_ended(self):
-        if self.q.is_empty(): self.pl.next_ended()
-        elif self.q.current is None: self.q.next()
+        if self.q.is_empty():
+            self.pl.next_ended()
+            self.q.sourced = False
+            self.pl.sourced = True
+        elif self.q.current is None:
+            self.q.next()
+            self.q.sourced = True
+            self.pl.sourced = False
 
     def previous(self):
         self.pl.previous()
@@ -64,6 +78,9 @@ class PlaylistMux(object):
 
     def reset(self):
         self.pl.reset()
+        self.q.go_to(None)
+        if not self.pl.is_empty():
+            self.pl.go_to(self.pl.get_iter((0,)))
 
     def enqueue(self, songs):
         for song in songs: self.q.append(row=[song])
@@ -71,9 +88,9 @@ class PlaylistMux(object):
 class PlaylistModel(gtk.ListStore):
     order = OFF
     repeat = False
+    sourced = False
     __iter = None
     __old_value = None
-    __sig = None
 
     __gsignals__ = {
         'songs-set': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
@@ -84,34 +101,26 @@ class PlaylistModel(gtk.ListStore):
         self.__played = []
 
     def set(self, songs):
-        if self.__sig is not None:
-            gobject.source_remove(self.__sig)
-            self.__sig = None
-
         oldsong = self.current
         if oldsong is None: oldsong = self.__old_value
         else: self.__old_value = oldsong
         self.__played = []
         self.__iter = None
-        self.clear()
         songs = songs[:]
-        if self.__set_idle(oldsong, songs):
-            self.__sig = gobject.idle_add(self.__set_idle, oldsong, songs)
+        copool.add(self.__set_idle, oldsong, songs)
+        copool.step(self.__set_idle)
 
     def __set_idle(self, oldsong, songs):
-        to_add = songs[:100]
-        del(songs[:100])
-        for song in to_add:
+        self.clear()
+        for count, song in enumerate(songs):
             iter = self.append(row=[song])
             if song == oldsong:
                 self.__iter = iter
-        if songs: return True
-        else:
-            if self.__iter is not None:
-                self.__old_value = None
-            self.__sig = None
-            self.emit('songs-set')
-            return False
+            if count and count % 100 == 0:
+                yield True
+        if self.__iter is not None:
+            self.__old_value = None
+        self.emit('songs-set')
 
     def remove(self, iter):
         if self.__iter and self[iter].path == self[self.__iter].path:
@@ -226,10 +235,12 @@ class PlaylistModel(gtk.ListStore):
         self.__iter = None
         if isinstance(song, gtk.TreeIter):
             self.__iter = song
+            self.sourced = True
         else:
             for row in self:
                 if row[0] == song:
                     self.__iter = row.iter
+                    self.sourced = True
                     break
 
     def find(self, song):
@@ -246,10 +257,10 @@ class PlaylistModel(gtk.ListStore):
         return not bool(len(self))
 
     def reset(self):
-        self.__played = []
         self.go_to(None)
-
-gobject.type_register(PlaylistModel)
+        # Must be done after go_to, since go_to can append to the
+        # played song list.
+        self.__played = []
 
 class SongList(AllTreeView, util.InstanceTracker):
     # A TreeView containing a list of songs.
@@ -259,18 +270,18 @@ class SongList(AllTreeView, util.InstanceTracker):
 
     CurrentColumn = None
 
-    class TextColumn(gtk.TreeViewColumn):
+    class TextColumn(qltk.views.TreeViewColumnButton):
         # Base class for other kinds of columns.
         _render = gtk.CellRendererText()
 
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                song = model[iter][0]
+                song = model.get_value(iter, 0)
                 cell.set_property('text', song.comma(tag))
             except AttributeError: pass
 
         def __init__(self, t):
-            gtk.TreeViewColumn.__init__(self, tag(t), self._render)
+            super(SongList.TextColumn, self).__init__(tag(t), self._render)
             self.header_name = t
             self.set_sizing(gtk.TREE_VIEW_COLUMN_GROW_ONLY)
             self.set_visible(True)
@@ -282,20 +293,19 @@ class SongList(AllTreeView, util.InstanceTracker):
         # The '~#' keys that are dates.
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                stamp = model[iter][0](tag)
-                if not stamp: cell.set_property('text', _("Never"))
+                stamp = model.get_value(iter, 0)(tag)
+                if not stamp:
+                    cell.set_property('text', _("Never"))
                 else:
                     date = datetime.datetime.fromtimestamp(stamp).date()
                     today = datetime.datetime.now().date()
                     days = (today - date).days
+                    if days == 0: format = "%X"
+                    elif days < 7: format = "%A"
+                    else: format = "%x"
                     stamp = time.localtime(stamp)
-                    if days == 0: rep = time.strftime("%X", stamp).decode(
-                        locale.getpreferredencoding())
-                    elif days < 7: rep = time.strftime("%A", stamp).decode(
-                        locale.getpreferredencoding())
-                    else: rep = time.strftime("%x", stamp).decode(
-                        locale.getpreferredencoding())
-                    cell.set_property('text', rep)
+                    text = time.strftime(format, stamp).decode(const.ENCODING)
+                    cell.set_property('text', text)
             except AttributeError: pass
 
     class WideTextColumn(TextColumn):
@@ -305,7 +315,7 @@ class SongList(AllTreeView, util.InstanceTracker):
         _render.set_property('ellipsize', pango.ELLIPSIZE_END)
 
         def __init__(self, tag):
-            SongList.TextColumn.__init__(self, tag)
+            super(SongList.WideTextColumn, self).__init__(tag)
             self.set_expand(True)
             self.set_resizable(True)
             self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
@@ -316,13 +326,13 @@ class SongList(AllTreeView, util.InstanceTracker):
         # a function call).
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                song = model[iter][0]
+                song = model.get_value(iter, 0)
                 cell.set_property(
                     'text', util.format_rating(song.get("~#rating", 0.5)))
             except AttributeError: pass
 
         def __init__(self):
-            SongList.TextColumn.__init__(self, "~#rating")
+            super(SongList.RatingColumn, self).__init__("~#rating")
             self.set_sizing(gtk.TREE_VIEW_COLUMN_GROW_ONLY)
 
             # Neither of TreeViewColumn or CellRendererText is a GTK
@@ -338,7 +348,7 @@ class SongList(AllTreeView, util.InstanceTracker):
         # Used for any tag without a '~' except 'title'.
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                song = model[iter][0]
+                song = model.get_value(iter, 0)
                 cell.set_property(
                     'text', song.get(tag, "").replace("\n", ", "))
             except AttributeError: pass
@@ -346,9 +356,9 @@ class SongList(AllTreeView, util.InstanceTracker):
     class FSColumn(WideTextColumn):
         # Contains text in the filesystem encoding, so needs to be
         # decoded safely (and also more slowly).
-        def _cdf(self, column, cell, model, iter, tag, code=util.fscoding):
+        def _cdf(self, column, cell, model, iter, tag, code=const.FSCODING):
             try:
-                song = model[iter][0]
+                song = model.get_value(iter, 0)
                 cell.set_property('text', util.unexpand(
                     song.comma(tag).decode(code, 'replace')))
             except AttributeError: pass
@@ -358,13 +368,13 @@ class SongList(AllTreeView, util.InstanceTracker):
         _render.set_property('xalign', 1.0)
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                song = model[iter][0]
+                song = model.get_value(iter, 0)
                 cell.set_property(
                     'text', util.format_time(song.get("~#length", 0)))
             except AttributeError: pass
 
         def __init__(self, tag="~#length"):
-            SongList.TextColumn.__init__(self, tag)
+            super(SongList.LengthColumn, self).__init__(tag)
             self.set_alignment(1.0)
 
     class NumericColumn(TextColumn):
@@ -376,24 +386,22 @@ class SongList(AllTreeView, util.InstanceTracker):
     class PatternColumn(WideTextColumn):
         def _cdf(self, column, cell, model, iter, tag):
             try:
-                song = model[iter][0]
-                cell.set_property('text', self.__pattern % song)
+                song = model.get_value(iter, 0)
+                cell.set_property('text', self._pattern % song)
             except AttributeError: pass
 
         def __init__(self, pattern):
-            SongList.WideTextColumn.__init__(self, pattern)
-            from parse import Pattern
-            self.__pattern = Pattern(pattern)
+            super(SongList.PatternColumn, self).__init__(_("pattern"))
+            self._pattern = Pattern(pattern)
 
-    def Menu(self, header, browser, watcher):
+    def Menu(self, header, browser, library):
         songs = self.get_selected_songs()
         if not songs: return
-        header = util.tagsplit(header)[0]
 
-        menu = browser.Menu(songs, self)
-        if menu is None: menu = gtk.Menu()
         can_filter = browser.can_filter
 
+        menu = browser.Menu(songs, self, library)
+
         def Filter(t):
             # Translators: The substituted string is the name of the
             # selected column (a translated tag name).
@@ -402,125 +410,31 @@ class SongList(AllTreeView, util.InstanceTracker):
             b.connect_object('activate', self.__filter_on, t, songs, browser)
             return b
 
-        if header == "~#rating":
-            item = gtk.MenuItem(_("_Rating"))
-            m2 = gtk.Menu()
-            item.set_submenu(m2)
-            for i in range(0, int(1.0/util.RATING_PRECISION)+1):
-                i *= util.RATING_PRECISION
-                itm = gtk.MenuItem("%0.2f\t%s" % (i, util.format_rating(i)))
-                m2.append(itm)
-                itm.connect_object(
-                    'activate', self.__set_rating, i, songs, watcher)
-            menu.append(item)
+        header = util.tagsplit(header)[0]
 
-        if (menu.get_children() and
-            not isinstance(menu.get_children()[-1], gtk.SeparatorMenuItem)):
-            menu.append(gtk.SeparatorMenuItem())
+        if can_filter("album") or can_filter("artist") or can_filter(header):
+            menu.preseparate()
 
-        if can_filter("artist"): menu.append(Filter("artist"))
-        if can_filter("album"): menu.append(Filter("album"))
+        if can_filter("artist"): menu.prepend(Filter("artist"))
+        if can_filter("album"): menu.prepend(Filter("album"))
         if (header not in ["artist", "album"] and can_filter(header)):
-            menu.append(Filter(header))
+            menu.prepend(Filter(header))
 
-        if (menu.get_children() and
-            not isinstance(menu.get_children()[-1], gtk.SeparatorMenuItem)):
-            menu.append(gtk.SeparatorMenuItem())
-
-        submenu = self.pm.create_plugins_menu(songs)
-        if submenu is not None:
-            b = gtk.ImageMenuItem(stock.PLUGINS)
-            menu.append(b)
-            b.set_submenu(submenu)
-
-            if (menu.get_children() and
-                not isinstance(menu.get_children()[-1],
-                               gtk.SeparatorMenuItem)):
-                menu.append(gtk.SeparatorMenuItem())
-
-        buttons = []
-
-        in_lib = True
-        can_add = True
-        is_file = True
-        for song in songs:
-            if song.get("~filename") not in library: in_lib = False
-            if not song.can_add: can_add = False
-            if not song.is_file: is_file = False
-
-
-        import browsers
-        try: playlists = browsers.playlists.Playlists.playlists()
-        except AttributeError: pass
-        else:            
-            b = qltk.MenuItem(_("_Add to Playlist"), gtk.STOCK_ADD)
-            menu.append(b)
-            b.set_sensitive(can_add)
-            submenu = gtk.Menu()
-            i = gtk.MenuItem(_("_New Playlist"))
-            i.connect('activate', self.__add_to_playlist, None, songs)
-            submenu.append(i)
-            submenu.append(gtk.SeparatorMenuItem())
-            submenu.set_size_request(int(i.size_request()[0] * 2), -1)
-
-            for playlist in playlists:
-                i = gtk.MenuItem(playlist.name)
-                i.child.set_ellipsize(pango.ELLIPSIZE_END)
-                i.connect('activate', self.__add_to_playlist, playlist, songs)
-                submenu.append(i)
-            b.set_submenu(submenu)
-        
-        b = gtk.ImageMenuItem(stock.ENQUEUE)
-        b.connect('activate', self.__enqueue, songs)
-        b.add_accelerator(
-            'activate', self.accelerators, ord('Q'), 0, gtk.ACCEL_VISIBLE)
-        menu.append(b)
-        buttons.append(b)
-        b.set_sensitive(can_add)
-
-        menu.append(gtk.SeparatorMenuItem())
-
-        b = gtk.ImageMenuItem(stock.REMOVE)
-        b.connect('activate', self.__remove, songs, watcher)
-        menu.append(b)
-        buttons.append(b)
-        b.set_sensitive(in_lib)
-
-        b = gtk.ImageMenuItem(gtk.STOCK_DELETE)
-        b.connect('activate', self.__delete, songs, watcher)
-        menu.append(b)
-        buttons.append(b)
-        b.set_sensitive(is_file)
-
-        b = gtk.ImageMenuItem(stock.EDIT_TAGS)
-        key, val = gtk.accelerator_parse("<alt>Return")
-        b.add_accelerator(
-            'activate', self.accelerators, key, val, gtk.ACCEL_VISIBLE)
-        b.connect_object('activate', SongProperties, watcher, songs)
-        menu.append(b)
-
-        b = gtk.ImageMenuItem(gtk.STOCK_INFO)
-        b.add_accelerator(
-            'activate', self.accelerators, ord('I'),
-            gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
-        b.connect_object('activate', Information, watcher, songs)
-        menu.append(b)
-
-        menu.connect_object('selection-done', gtk.Menu.destroy, menu)
+        item = gtk.MenuItem(_("_Rating"))
+        m2 = gtk.Menu()
+        item.set_submenu(m2)
+        for i in range(0, int(1.0/util.RATING_PRECISION)+1):
+            i *= util.RATING_PRECISION
+            itm = gtk.MenuItem("%0.2f\t%s" % (i, util.format_rating(i)))
+            m2.append(itm)
+            itm.connect_object(
+                'activate', self.__set_rating, i, songs, library)
+        menu.preseparate()
+        menu.prepend(item)
         menu.show_all()
         return menu
 
-    def __add_to_playlist(self, activator, playlist, songs):
-        import browsers
-        if playlist is None:
-            if len(songs) == 1: title = songs[0].comma("title")
-            else: title = _("%(title)s and %(count)d more") % (
-                {'title':songs[0].comma("title"), 'count':len(songs) - 1})
-            playlist = browsers.playlists.Playlist.new(title)
-        playlist.extend(songs)
-        browsers.playlists.Playlists.changed(playlist)
-
-    def __init__(self, watcher):
+    def __init__(self, library, player=None):
         super(SongList, self).__init__()
         self._register_instance(SongList)
         self.set_model(PlaylistModel())
@@ -528,21 +442,26 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.set_rules_hint(True)
         self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         self.set_column_headers(self.headers)
-        sigs = [watcher.connect('changed', self.__song_updated),
-                watcher.connect('removed', self.__song_removed),
-                watcher.connect('paused', self.__redraw_current),
-                watcher.connect('unpaused', self.__redraw_current)
-                ]
+        librarian = library.librarian
+        sigs = [librarian.connect('changed', self.__song_updated),
+                librarian.connect('removed', self.__song_removed)]
         for sig in sigs:
-            self.connect_object('destroy', watcher.disconnect, sig)
+            self.connect_object('destroy', librarian.disconnect, sig)
+        if player:
+            sigs = [player.connect('paused', self.__redraw_current),
+                    player.connect('unpaused', self.__redraw_current)]
+            for sig in sigs:
+                self.connect_object('destroy', player.disconnect, sig)
 
-        self.connect('button-press-event', self.__button_press, watcher)
-        self.connect('key-press-event', self.__key_press, watcher)
+        self.connect('button-press-event', self.__button_press, librarian)
+        self.connect('key-press-event', self.__key_press, librarian)
 
         self.disable_drop()
         self.connect('drag-motion', self.__drag_motion)
-        self.connect('drag-data-get', self.__drag_data_get, watcher)
-        self.connect('drag-data-received', self.__drag_data_received)
+        self.connect(
+            'drag-leave', lambda s, ctx, time: s.parent.drag_unhighlight())
+        self.connect('drag-data-get', self.__drag_data_get)
+        self.connect('drag-data-received', self.__drag_data_received, library)
 
         # Enabling this screws up rating and enqueuing
         #self.set_search_column(0)
@@ -551,20 +470,19 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.accelerators = gtk.AccelGroup()
         key, mod = gtk.accelerator_parse("<alt>Return")
         self.accelerators.connect_group(
-            key, mod, 0, lambda *args: self.__song_properties(watcher))
+            key, mod, 0, lambda *args: self.__song_properties(librarian))
         key, mod = gtk.accelerator_parse("<control>I")
         self.accelerators.connect_group(
-            key, mod, 0, lambda *args: self.__information(watcher))
-        self.accelerators.connect
+            key, mod, 0, lambda *args: self.__information(librarian))
 
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
-            value = model[iter][0](column.header_name)
+            value = model.get_value(iter, 0)(column.header_name)
             if not isinstance(value, basestring): continue
             elif key in value.lower() or key in value: return False
         else: return True
 
-    def enable_drop(self):
+    def enable_drop(self, by_row=True):
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
                    ("text/uri-list", 0, 2)]
         self.drag_source_set(
@@ -572,6 +490,7 @@ class SongList(AllTreeView, util.InstanceTracker):
             gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
         self.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,
                            gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
+        self.__drop_by_row = by_row
 
     def disable_drop(self):
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
@@ -581,21 +500,26 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.drag_dest_unset()
 
     def __drag_motion(self, view, ctx, x, y, time):
-        try: self.set_drag_dest_row(*self.get_dest_row_at_pos(x, y))
-        except TypeError:
-            if len(self.get_model()) == 0: path = 0
-            else: path = len(self.get_model()) - 1
-            self.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_AFTER)
-        if ctx.get_source_widget() == self: kind = gtk.gdk.ACTION_MOVE
-        else: kind = gtk.gdk.ACTION_COPY
-        ctx.drag_status(kind, time)
-        return True
+        if self.__drop_by_row:
+            try: self.set_drag_dest_row(*self.get_dest_row_at_pos(x, y))
+            except TypeError:
+                if len(self.get_model()) == 0: path = 0
+                else: path = len(self.get_model()) - 1
+                self.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_AFTER)
+            if ctx.get_source_widget() == self: kind = gtk.gdk.ACTION_MOVE
+            else: kind = gtk.gdk.ACTION_COPY
+            ctx.drag_status(kind, time)
+            return True
+        else:
+            self.parent.drag_highlight()
+            ctx.drag_status(gtk.gdk.ACTION_COPY, time)
+            return True
 
     def __drag_data_delete(self, view, ctx):
         map(view.get_model(), self.__drag_iters)
         self.__drag_iters = []
 
-    def __drag_data_get(self, view, ctx, sel, tid, etime, watcher):
+    def __drag_data_get(self, view, ctx, sel, tid, etime):
         model, paths = self.get_selection().get_selected_rows()
         if tid == 1:
             songs = [model[path][0] for path in paths
@@ -607,10 +531,8 @@ class SongList(AllTreeView, util.InstanceTracker):
                       "song lists or the queue.")).run()
                 ctx.drag_abort(etime)
                 return
-            added = filter(library.add_song, songs)
             filenames = [song("~filename") for song in songs]
             sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
-            if added: watcher.added(added)
             if ctx.action == gtk.gdk.ACTION_MOVE:
                 self.__drag_iters = map(model.get_iter, paths)
             else: self.__drag_iters = []
@@ -619,7 +541,13 @@ class SongList(AllTreeView, util.InstanceTracker):
             sel.set_uris(uris)
             self.__drag_iters = []
 
-    def __drag_data_received(self, view, ctx, x, y, sel, info, etime):
+    def __drag_data_browser_dropped(self, songs):
+        window = qltk.get_top_parent(self)
+        if callable(window.browser.dropped):
+            return window.browser.dropped(self, songs)
+        else: return False
+
+    def __drag_data_received(self, view, ctx, x, y, sel, info, etime, library):
         model = view.get_model()
         if info == 1:
             filenames = sel.data.split("\x00")
@@ -635,11 +563,23 @@ class SongList(AllTreeView, util.InstanceTracker):
             ctx.finish(False, False, etime)
             return
 
+        to_add = []
+        for filename in filenames:
+            if filename not in library.librarian:
+                library.add_filename(filename)
+            elif filename not in library:
+                to_add.append(library.librarian[filename])
+        library.add(to_add)
         songs = filter(None, map(library.get, filenames))
         if not songs:
             ctx.finish(bool(not filenames), False, etime)
             return
 
+        if not self.__drop_by_row:
+            success = self.__drag_data_browser_dropped(songs)
+            ctx.finish(success, False, etime)
+            return
+
         try: path, position = view.get_dest_row_at_pos(x, y)
         except TypeError:
             path = max(0, len(model) - 1)
@@ -681,7 +621,7 @@ class SongList(AllTreeView, util.InstanceTracker):
             for song in songs: values.update(song.list(header))
         browser.filter(header, list(values))
 
-    def __button_press(self, view, event, watcher):
+    def __button_press(self, view, event, librarian):
         if event.button != 1: return
         x, y = map(int, [event.x, event.y])
         try: path, col, cellx, celly = view.get_path_at_pos(x, y)
@@ -696,47 +636,32 @@ class SongList(AllTreeView, util.InstanceTracker):
             rating = max(0.0, min(1.0, count * util.RATING_PRECISION))
             if (rating <= util.RATING_PRECISION and
                 song["~#rating"] == util.RATING_PRECISION): rating = 0
-            self.__set_rating(rating, [song], watcher)
+            self.__set_rating(rating, [song], librarian)
 
-    def __remove(self, item, songs, watcher):
-        # User requested that the selected songs be removed.
-        map(library.remove, songs)
-        watcher.removed(songs)
+    def __set_rating(self, value, songs, librarian):
+        for song in songs:
+            song["~#rating"] = value
+        librarian.changed(songs)
 
-    def __enqueue(self, item, songs):
-        songs = filter(lambda s: s.can_add, songs)
-        if songs:
-            from widgets import main, watcher
-            added = filter(library.add_song, songs)
-            main.playlist.enqueue(songs)
-            if added: watcher.added(added)
-
-    def __delete(self, item, songs, watcher):
-        files = [song["~filename"] for song in songs]
-        d = DeleteDialog(self, files)
-        removed = d.run()
-        d.destroy()
-        removed = filter(None, map(library.get, removed))
-        if removed:
-            map(library.remove, removed)
-            watcher.removed(removed)
-
-    def __set_rating(self, value, songs, watcher):
-        for song in songs: song["~#rating"] = value
-        watcher.changed(songs)
-
-    def __key_press(self, songlist, event, watcher):
+    def __key_press(self, songlist, event, librarian):
         if event.string in ['0', '1', '2', '3', '4']:
             rating = min(1.0, int(event.string) * util.RATING_PRECISION)
-            self.__set_rating(rating, self.get_selected_songs(), watcher)
+            self.__set_rating(rating, self.get_selected_songs(), librarian)
         elif event.string in ['Q', 'q']:
-            self.__enqueue(None, self.get_selected_songs())
+            self.__enqueue(self.get_selected_songs())
 
-    def __redraw_current(self, watcher, song=None):
+    def __enqueue(self, songs):
+        songs = filter(lambda s: s.can_add, songs)
+        if songs:
+            from widgets import main
+            main.playlist.enqueue(songs)
+
+    def __redraw_current(self, player, song=None):
         iter = self.model.current_iter
         if iter: self.model.row_changed(self.model.get_path(iter), iter)
 
     def set_all_column_headers(cls, headers):
+        config.set("settings", "headers", " ".join(headers))
         try: headers.remove("~current")
         except ValueError: pass
         cls.headers = headers
@@ -754,9 +679,14 @@ class SongList(AllTreeView, util.InstanceTracker):
     def get_sort_by(self):
         for header in self.get_columns():
             if header.get_sort_indicator():
-                return (header.header_name,
-                        header.get_sort_order() == gtk.SORT_DESCENDING)
-        else: return "artist", False
+                try: tag = header._pattern.format
+                except AttributeError: tag = header.header_name
+                sort = header.get_sort_order()
+                return (tag, sort == gtk.SORT_DESCENDING)
+        else: return "album", False
+
+    def is_sorted(self):
+        return max([col.get_sort_indicator() for col in self.get_columns()])
 
     # Resort based on the header clicked.
     def set_sort_by(self, header, tag=None, order=None, refresh=True):
@@ -797,10 +727,12 @@ class SongList(AllTreeView, util.InstanceTracker):
             elif tag == "~length": tag = "~#length"
             elif tag == "~album~part": tag = "album"
 
-            songs = [(song(tag), song.sort_key, song) for song in songs]
-            songs.sort()
-            if reverse: songs.reverse()
-            songs = [song[2] for song in songs]
+            if callable(tag):
+                # A pattern is currently selected.
+                sort_func = lambda song: (tag(song), song.sort_key, song)
+            else:
+                sort_func = lambda song: (song(tag), song.sort_key, song)
+            songs.sort(key=sort_func, reverse=reverse)
         else:
             self.set_sort_by(None, refresh=False)
 
@@ -824,31 +756,38 @@ class SongList(AllTreeView, util.InstanceTracker):
         model, rows = self.get_selection().get_selected_rows()
         return [model[row][0] for row in rows]
 
-    def __song_updated(self, watcher, songs):
+    def __song_updated(self, librarian, songs):
         model = self.get_model()
         for row in model:
-            if row[0] in songs: model.row_changed(row.path, row.iter)
+            if row[0] in songs:
+                model.row_changed(row.path, row.iter)
 
-    def __song_removed(self, watcher, songs):
+    def __song_removed(self, librarian, songs):
         # The selected songs are removed from the library and should
         # be removed from the view.
         map(self.model.remove, self.model.find_all(songs))
 
-    def __song_properties(self, watcher):
+    def __song_properties(self, librarian):
         model, rows = self.get_selection().get_selected_rows()
-        if rows: songs = [model[row][0] for row in rows]
+        if rows:
+            songs = [model[row][0] for row in rows]
         else:
             from player import playlist
-            songs = [playlist.song]
-        SongProperties(watcher, songs)
+            if playlist.song:
+                songs = [playlist.song]
+            else: return
+        SongProperties(librarian, songs)
 
-    def __information(self, watcher):
+    def __information(self, librarian):
         model, rows = self.get_selection().get_selected_rows()
-        if rows: songs = [model[row][0] for row in rows]
+        if rows:
+            songs = [model[row][0] for row in rows]
         else:
             from player import playlist
-            songs = [playlist.song]
-        Information(watcher, songs)
+            if playlist.song:
+                songs = [playlist.song]
+            else: return
+        Information(librarian, songs)
 
     # Build a new filter around our list model, set the headers to their
     # new values.
@@ -869,11 +808,110 @@ class SongList(AllTreeView, util.InstanceTracker):
             elif t.startswith("~#"): column = self.NumericColumn(t)
             elif t in ["~filename", "~basename", "~dirname"]:
                 column = self.FSColumn(t)
-            elif "~" not in t and t != "title":
-                column = self.NonSynthTextColumn(t)
             elif t.startswith("<"):
                 column = self.PatternColumn(t)
+            elif "~" not in t and t != "title":
+                column = self.NonSynthTextColumn(t)
             else: column = self.WideTextColumn(t)
             column.connect('clicked', self.set_sort_by)
+            column.connect('button-press-event', self.__showmenu)
+            column.connect('popup-menu', self.__showmenu)
             column.set_reorderable(True)
             self.append_column(column)
+
+    def __getmenu(self, column):
+        menu = gtk.Menu()
+        menu.connect_object('selection-done', gtk.Menu.destroy, menu)
+
+        current = SongList.headers[:]
+        current_set = set(current)
+        current = zip(map(util.tag, current), current)
+        tips = qltk.Tooltips(menu)
+
+        def add_header_toggle(menu, (header, tag), active,
+                column=column, tips=tips):
+            item = gtk.CheckMenuItem(header)
+            item.tag = tag
+            item.set_active(active)
+            item.connect('activate', self.__toggle_header_item, column)
+            item.show()
+            tips.set_tip(item, tag, tag)
+            menu.append(item)
+
+        for header in current:
+            add_header_toggle(menu, header, True)
+
+        sep = gtk.SeparatorMenuItem()
+        sep.show()
+        menu.append(sep)
+
+        trackinfo = """title genre ~title~version ~#track
+            ~#playcount ~#skipcount ~#rating ~#length""".split()
+        peopleinfo = """artist ~people performer arranger author composer
+            conductor lyricist originalartist""".split()
+        albuminfo = """album ~album~part labelid ~#disc ~#discs
+            ~#tracks albumartist""".split()
+        dateinfo = """date originaldate recordingdate ~#laststarted
+            ~#lastplayed ~#added ~#mtime""".split()
+        fileinfo = """~format ~#bitrate ~filename ~basename ~dirname
+            ~uri""".split()
+        copyinfo = """copyright organization location isrc
+            contact website""".split()
+
+        for name, group in [
+            (_("_Track Headers"), trackinfo),
+            (_("_Album Headers"), albuminfo),
+            (_("_People Headers"), peopleinfo),
+            (_("_Date Headers"), dateinfo),
+            (_("_File Headers"), fileinfo),
+            (_("_Production Headers"), copyinfo),
+        ]:
+            item = gtk.MenuItem(name)
+            item.show()
+            menu.append(item)
+            submenu = gtk.Menu()
+            item.set_submenu(submenu)
+            for header in sorted(zip(map(tag, group), group)):
+                add_header_toggle(submenu, header, header[1] in current_set)
+
+        sep = gtk.SeparatorMenuItem()
+        sep.show()
+        menu.append(sep)
+        custom = gtk.MenuItem(_("_Customize Headers..."))
+        custom.show()
+        custom.connect('activate', self.__add_custom_column)
+        menu.append(custom)
+
+        return menu
+
+    def __toggle_header_item(self, item, column):
+        headers = SongList.headers[:]
+        if item.get_active():
+            try: headers.insert(self.get_columns().index(column), item.tag)
+            except ValueError: headers.append(item.tag)
+        else:
+            try: headers.remove(item.tag)
+            except ValueError: pass
+
+        SongList.set_all_column_headers(headers)
+        SongList.headers = headers
+
+    def __add_custom_column(self, item):
+        # Prefs has to import SongList, so do this here to avoid
+        # a circular import.
+        from qltk.prefs import PreferencesWindow
+        win = PreferencesWindow(self)
+        win.child.set_current_page(0)
+        
+    def __showmenu(self, column, event=None):
+        time = gtk.get_current_event_time()
+        if event is not None and event.button != 3:
+            return
+
+        if event:
+            self.__getmenu(column).popup(None, None, None, event.button, time)
+            return True
+
+        widget = column.get_widget()
+        return qltk.popup_menu_under_widget(self.__getmenu(column),
+                widget, 3, time)
