@@ -8,44 +8,60 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 #
-# $Id: quodlibet.py 2917 2006-03-03 06:26:48Z piman $
+# $Id: quodlibet.py 3716 2006-07-29 23:26:33Z piman $
+
+import os
+import signal
+import sys
+import tempfile
+import time
+
+import config
+import const
+import util
+
+from threading import Thread
+
+from util import to
 
-import os, sys
 global play
 play = False
 
 def main():
-    import player
-    import util; util.mkdir(const.DIR)
-    import signal, widgets
+    # Load configuration data and scan the library for new/changed songs.
+    config.init(const.CONFIG)
+    library = load_library()
+    player = load_player(library)
+
+    import widgets
+
+    util.mkdir(const.USERDIR)
     SIGNALS = [signal.SIGINT, signal.SIGTERM, signal.SIGHUP]
 
     try: ratings = config.getint("settings", "ratings")
     except (ValueError, TypeError): pass
     else: util.RATING_PRECISION = 1.0/ratings
 
-    locale.getlocale(locale.LC_NUMERIC)
-    window = widgets.init()
+    window = widgets.init(player, library)
     if "--debug" not in sys.argv:
-        enable_periodic_save()
-        gtk.quit_add(1, widgets.save_library, window, player.playlist)
-    for sig in SIGNALS: signal.signal(sig, gtk.main_quit)
+        enable_periodic_save(library)
+        gtk.quit_add(1, widgets.save_library, window, player, library)
+    for sig in SIGNALS:
+        signal.signal(sig, gtk.main_quit)
     gtk.threads_init()
-    if play: player.playlist.paused = False
+    if play: player.paused = False
     gtk.main()
 
 def print_fifo(command):
     if not os.path.exists(const.CURRENT):
         raise SystemExit("not-running")
     else:
-        from tempfile import mkstemp
-        fd, filename = mkstemp()
+        fd, filename = tempfile.mkstemp()
         try:
             os.unlink(filename)
             # mkfifo fails if the file exists, so this is safe.
             os.mkfifo(filename, 0600)
 
-            import signal
             signal.signal(signal.SIGALRM, lambda: "" + 2)
             signal.alarm(1)
             f = file(const.CONTROL, "w")
@@ -64,23 +80,10 @@ def print_fifo(command):
             except EnvironmentError: pass
             raise SystemExit("not-running")
 
-def refresh_cache():
-    if isrunning():
-        raise SystemExit(to(_(
-            "The library cannot be refreshed while Quod Libet is running.")))
-    import library, config, const
-    config.init(const.CONFIG)
-    lib = library.init()
-    print to(_("Loading, scanning, and saving your library."))
-    lib.load(const.LIBRARY)
-    for x, y in lib.rebuild(): pass
-    lib.save(const.LIBRARY)
-    raise SystemExit
-
-def print_playing(fstring = "<artist~album~tracknumber~title>"):
-    import util
-    from parse import Pattern
+def print_playing(fstring="<artist~album~tracknumber~title>"):
     from formats._audio import AudioFile
+    from parse import Pattern
+
     try:
         fn = file(const.CURRENT)
         data = {}
@@ -110,7 +113,6 @@ def control(c):
         raise SystemExit(to(_("Quod Libet is not running.")))
     else:
         try:
-            import signal
             # This is a total abuse of Python! Hooray!
             signal.signal(signal.SIGALRM, lambda: "" + 2)
             signal.alarm(1)
@@ -122,17 +124,17 @@ def control(c):
             print to(_("Unable to write to %s. Removing it.") % const.CONTROL)
             try: os.unlink(const.CONTROL)
             except OSError: pass
-            if c != 'focus': raise SystemExit(True)
+            if c != 'focus':
+                raise SystemExit(True)
         else:
             raise SystemExit
 
-def enable_periodic_save():
+def enable_periodic_save(library):
     # Check every 5 minutes to see if the library/config on disk are
     # over 15 minutes old; if so, update them. This function can, in theory,
     # break if saving the library takes more than 5 minutes.
-    import gobject, time, util
-    from library import library
-    from threading import Thread
+    import gobject
+
     def save():
         if (time.time() - util.mtime(const.LIBRARY)) > 15*60:
             library.save(const.LIBRARY)
@@ -152,13 +154,11 @@ def process_arguments():
                     "set-rating", "set-browser", "open-browser", "random",
                     "song-list", "queue", "enqueue"]
 
-    from util import OptionParser
-    options = OptionParser(
+    options = util.OptionParser(
         "Quod Libet", const.VERSION, 
         _("a music library and player"),
-        _("[ --refresh-library | --print-playing | control ]"))
+        _("[ --print-playing | control ]"))
 
-    options.add("refresh-library", help=_("Rescan your library and exit"))
     options.add("print-playing", help=_("Print the playing song and exit"))
     options.add("start-playing", help=_("Begin playing immediately"))
 
@@ -187,18 +187,22 @@ def process_arguments():
         ("repeat", _("Turn repeat off, on, or toggle it"), "0|1|t"),
         ("volume", _("Set the volume"), "+|-|0..100"),
         ("query", _("Search your audio library"), _("query")),
-        ("play-file", _("Play a file"), _("filename")),
+        ("play-file", _("Play a file"), Q_("command|filename")),
         ("set-rating", _("Rate the playing song"), "0.0..1.0"),
         ("set-browser", _("Set the current browser"), "BrowserName"),
         ("open-browser", _("Open a new browser"), "BrowserName"),
         ("queue", _("Show or hide the queue"), "on|off|t"),
         ("song-list", _("Show or hide the main song list"), "on|off|t"),
-        ("random", _("Filter on a random value"), _("tag")),
+        ("random", _("Filter on a random value"), Q_("command|tag")),
         ("filter", _("Filter on a tag value"), _("tag=value")),
         ("enqueue", _("Enqueue a file or query"), "%s|%s" %(
-        _("filename"),_( "query"))),
+        Q_("command|filename"), _("query"))),
         ]: options.add(opt, help=help, arg=arg)
 
+    options.add("sm-config-prefix", arg="dummy")
+    options.add("sm-client-id", arg="prefix")
+    options.add("screen", arg="dummy")
+
     def is_time(str):
         if str[0] not in "+-0123456789": return False
         elif str[0] in "+-": str = str[1:]
@@ -222,8 +226,7 @@ def process_arguments():
     opts, args = options.parse()
 
     for command, arg in opts.items():
-        if command == "refresh-library": refresh_cache()
-        elif command in controls: control(command)
+        if command in controls: control(command)
         elif command in controls_opt:
             if command in validators and not validators[command](arg):
                 sys.stderr.write(
@@ -249,20 +252,16 @@ def process_arguments():
 
 def load_library():
     import library
-    library.init(const.LIBRARY)
+    lib = library.init(const.LIBRARY)
     print to(_("Loaded song library."))
-    from library import library
+    return lib
 
-    if config.get("settings", "scan"):
-        for a, c, r in library.scan(config.get("settings", "scan").split(":")):
-            pass
-
-def load_player():
+def load_player(library):
     # Try to initialize the playlist and audio output.
     print to(_("Opening audio device."))
     import player
     sink = config.get("settings", "pipeline")
-    try: player.init(sink)
+    try: playlist = player.init(sink, library.librarian)
     except player.NoSinkError:
         import widgets, gobject
         gobject.idle_add(widgets.no_sink_quit, sink)
@@ -275,46 +274,28 @@ def load_player():
         gtk.main()
         config.write(const.CONFIG)
         raise SystemExit(True)
+    else: return playlist
 
 if __name__ == "__main__":
     basedir = os.path.dirname(os.path.realpath(__file__))
-    if not os.path.exists(os.path.join(basedir, "quodlibet.py")):
-        if os.path.exists(os.path.join(os.getcwd(), "quodlibet.py")):
-            basedir = os.getcwd()
     if basedir.endswith("/share/quodlibet"):
         sys.path.append(basedir[:-15] + "lib/quodlibet")
-    i18ndir = "/usr/share/locale"
 
-    import locale, gettext, util
-    try: locale.setlocale(locale.LC_ALL, '')
-    except: pass
-
-    gettext.bindtextdomain("quodlibet")
-    gettext.textdomain("quodlibet")
-    util.gettext_install("quodlibet", unicode=True)
+    util.python_init()
+    util.gettext_install()
     util.ctypes_init()
 
-    from util import to
-    import const
     if "--debug" not in sys.argv:
         process_arguments()
-        if os.path.exists(const.CONTROL):
+        if isrunning():
             print to(_("Quod Libet is already running."))
             control('focus')
 
-    # Get to the right directory for our data.
-    os.chdir(basedir)
-    # Initialize GTK.
-    import pygtk
-    pygtk.require('2.0')
+    # GTK+ eats command line arguments and babies, so we have to delay
+    # imports until at least this late.
+    util.gtk_init()
+    import gtk
     import pygst
     pygst.require('0.10')
-    import gtk
-    util.gtk_init()
 
-    # Load configuration data and scan the library for new/changed songs.
-    import config
-    config.init(const.CONFIG)
-    load_library()
-    load_player()
     main()
