@@ -4,29 +4,57 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 #
-# $Id: __init__.py 2903 2006-02-26 21:53:08Z piman $
+# $Id: __init__.py 3793 2006-08-16 06:25:50Z piman $
 
-import os, sys, sre, locale
+import gettext
+import locale
+import os
+import sre
+import sys
 
-def gettext_install(domain, localedir=None, unicode=False):
-    import gettext, __builtin__
-    t = gettext.translation(domain, localedir, fallback=True)
-    if unicode: gettext, ngettext = t.ugettext, t.ungettext
-    else: gettext, ngettext = t.gettext, t.ngettext
-    __builtin__.__dict__["_"] = gettext
-    __builtin__.__dict__["ngettext"] = ngettext
+from const import FSCODING as fscoding, ENCODING
+from util.i18n import GlibTranslations
+
+def gettext_install():
+    try: locale.setlocale(locale.LC_ALL, '')
+    except locale.Error: pass
+    try:
+        t = gettext.translation("quodlibet", class_=GlibTranslations)
+    except IOError:
+        t = GlibTranslations()
+
+    t.install()
+
+def python_init():
+    sre.escape = re_esc
+
+def re_esc(str, BAD="/.^$*+?{,\\[]|()<>#=!:"):
+    needs_escape = lambda c: (c in BAD and "\\" + c) or c
+    return "".join(map(needs_escape, str))
 
 def gtk_init():
-    import gtk, gobject
-    if gtk.pygtk_version > (2, 8):
-        gobject.type_register = lambda Kind: True
-
+    import pygtk
+    pygtk.require('2.0')
+    import gtk
     # http://bugzilla.gnome.org/show_bug.cgi?id=318953
     if gtk.gtk_version < (2, 8, 8):
         class TVProxy(gtk.TreeView):
             def set_search_equal_func(self, func, *args): pass
         gtk.TreeView = TVProxy
 
+def listdir(path, hidden=False):
+    """List files in a directory, sorted, fully-qualified.
+
+    If hidden is false, Unix-style hidden files are not returned.
+    """
+    if hidden: filt = None
+    else: filt = lambda base: not base.startswith(".")
+    if path.endswith(os.sep): join = "".join
+    else: join = os.sep.join
+    return [join([path, basename])
+            for basename in sorted(os.listdir(path))
+            if filt(basename)]
+
 def ctypes_init():
     try: import ctypes
     except ImportError: pass
@@ -114,8 +142,7 @@ class OptionParser(object):
         if self.__description:
             s += "%s - %s\n" % (self.__name, self.__description)
         s += "\n"
-        keys = self.__help.keys()
-        keys.sort()
+        keys = sorted(self.__help.keys())
         try: keys.remove("help")
         except ValueError: pass
         try: keys.remove("version")
@@ -180,9 +207,10 @@ def to(string, frm="utf-8"):
     """Convert a string to the system encoding; used if you need to
     print to stdout. If you pass in a str (rather than a unicode) you
     should specify the encoding it's in with 'frm'."""
-    enc = locale.getpreferredencoding()
-    if isinstance(string, unicode): return string.encode(enc, "replace")
-    else: return string.decode(frm).encode(enc, "replace")
+    if isinstance(string, unicode):
+        return string.encode(ENCODING, "replace")
+    else:
+        return string.decode(frm).encode(ENCODING, "replace")
 
 def mtime(filename):
     """Return the mtime of a file, or 0 if an error occurs."""
@@ -209,12 +237,7 @@ def unescape(str):
     """Unescape a string in a manner suitable for XML/Pango."""
     return str.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
 
-def re_esc(str):
-    return "".join(map(
-        lambda a: (a in "/.^$*+?{,}\\[]|()<>#=!:" and "\\" + a) or a, str))
-sre.escape = re_esc
-
-def parse_time(timestr):
+def parse_time(timestr, err=(ValueError, sre.error)):
     """Parse a time string in hh:mm:ss, mm:ss, or ss format."""
     if timestr[0:1] == "-":
         m = -1
@@ -224,8 +247,7 @@ def parse_time(timestr):
     try:
         return m * reduce(lambda s, a: s * 60 + int(a),
                           sre.split(r":|\.", timestr), 0)
-    except (ValueError, sre.error):
-        return 0
+    except err: return 0
 
 RATING_PRECISION = 0.25
 def format_rating(value):
@@ -279,14 +301,6 @@ def format_time_long(time):
     if len(time_str) > 2: time_str.pop()
     return ", ".join(time_str)
 
-# http://developer.gnome.org/doc/API/2.0/glib/glib-running.html
-if "G_FILENAME_ENCODING" in os.environ:
-    fscoding = os.environ["G_FILENAME_ENCODING"].split(",")[0]
-    if fscoding == "@locale": fscoding = locale.getpreferredencoding()
-elif "G_BROKEN_FILENAMES" in os.environ:
-    fscoding = locale.getpreferredencoding()
-else: fscoding = "utf-8"
-
 def fsdecode(s):
     """Decoding a string according to the filesystem encoding."""
     if isinstance(s, unicode): return s
@@ -411,24 +425,28 @@ def find_subtitle(title):
                 return title.rstrip(), subtitle
     else: return title, None
 
-def unexpand(filename):
+def unexpand(filename, HOME=os.path.expanduser("~")):
     """Replace the user's home directory with ~/, if it appears at the
     start of the path name."""
-    if filename == os.path.expanduser("~"): return "~"
-    elif filename.startswith(os.path.expanduser("~/")):
-        filename = filename.replace(os.path.expanduser("~/"), "~/", 1)
+    if filename == HOME: return "~"
+    elif filename.startswith(HOME + "/"):
+        filename = filename.replace(HOME, "~", 1)
     return filename
 
 def website(site):
     site = site.replace("\\", "\\\\").replace("\"", "\\\"")
-    for s in (["sensible-browser", "gnome-open"] +
+    for prog in (["sensible-browser", "gnome-open"] +
               os.environ.get("BROWSER","").split(":")):
-        if iscommand(s):
-            if "%s" in s:
-                s = s.replace("%s", '"' + site + '"')
-                s = s.replace("%%", "%")
-            else: s += " \"%s\"" % site
-            if os.system(s + " &") == 0: return True
+        if iscommand(prog):
+            args = prog.split()
+            for i, arg in enumerate(args):
+                if arg == "%s":
+                    args[i] = site
+                    break
+            else: args.append(site)
+            try: spawn(args)
+            except RuntimeError: return False
+            else: return True
     else: return False
 
 def tag(name, cap=True):
@@ -445,7 +463,15 @@ def tag(name, cap=True):
             except IndexError: return _("Invalid tag")
             else: return _(HEADERS_FILTER.get(tag, tag.replace("_", " ")))
         parts = map(readable, tagsplit(name))
-        if cap: parts = map(capitalize, parts)
+        if cap:
+            # Translators: If tag names, when capitalized, should not
+            # be title-cased ("Looks Like This"), but rather only have
+            # the first letter capitalized, translate this string as
+            # something other than "check|titlecase?".
+            if _("check|titlecase?") == "check|titlecase?":
+                parts = map(title, parts)
+            else:
+                parts = map(capitalize, parts)
         return " / ".join(parts)
 
 def tagsplit(tag):
@@ -463,17 +489,65 @@ def tagsplit(tag):
         return tags
     else: return [tag]
 
-HEADERS_FILTER = { "tracknumber": "track",
-                   "discnumber": "disc",
-                   "labelid": "label ID",
-                   "bpm": "BPM",
-                   "isrc": "ISRC",
-                   "lastplayed": "last played",
-                   "filename": "full name",
-                   "playcount": "plays",
-                   "skipcount": "skips",
-                   "mtime": "modified",
-                   "mountpoint": "mount point",
-                   "basename": "filename",
-                   "dirname": "directory" }
+def spawn(argv, stdout=False):
+    """Asynchronously run a program. argv[0] is the executable name, which
+    must be fully qualified or in the path. If stdout is True, return
+    a file object corresponding to the child's standard output; otherwise,
+    return the child's process ID.
 
+    argv must be strictly str objects to avoid encoding confusion."""
+
+    import gobject
+    types = map(type, argv)
+    if not (min(types) == max(types) == str):
+        raise TypeError("executables and arguments must be str objects")
+    args = gobject.spawn_async(
+        argv, flags=gobject.SPAWN_SEARCH_PATH, standard_output=stdout)
+    if stdout: return os.fdopen(args[2])
+    else: return args[0]
+
+N_ = lambda value: value
+
+HEADERS_FILTER = {
+    "tracknumber": N_("track"),
+    "discnumber": N_("disc"),
+    "labelid": N_("label ID"),
+    "bpm": N_("BPM"),
+    "isrc": "ISRC",
+    "lastplayed": N_("last played"),
+    "laststarted": N_("last started"),
+    "filename": N_("full name"),
+    "playcount": N_("plays"),
+    "skipcount": N_("skips"),
+    "mtime": N_("modified"),
+    "mountpoint": N_("mount point"),
+    "basename": N_("filename"),
+    "dirname": N_("directory"),
+    "uri": "URI",
+
+    # http://musicbrainz.org/doc/MusicBrainzTag
+    "musicbrainz_trackid": N_("MusicBrainz track ID"),
+    "musicbrainz_albumid": N_("MusicBrainz album ID"),
+    "musicbrainz_artistid": N_("MusicBrainz artist ID"),
+    "musicbrainz_albumartistid": N_("MusicBrainz album artist ID"),
+    "musicbrainz_trmid": N_("MusicBrainz TRM ID"),
+    "musicip_puid": N_("MusicIP PUID"),
+    "musicbrainz_albumstatus": N_("MusicBrainz album status"),
+    "musicbrainz_albumtype": N_("MusicBrainz album type"),
+
+    # Translators: A volume adjustment, not "to get/acquire".
+    "replaygain_track_gain": N_("track gain"),
+    "replaygain_track_peak": N_("track peak"),
+    # Translators: A volume adjustment, not "to get/acquire".
+    "replaygain_album_gain": N_("album gain"),
+    "replaygain_album_peak": N_("album peak"),
+
+    "albumartist": N_("album artist"),
+    "originaldate": N_("original release date"),
+    "originalalbum": N_("original album"),
+    "originalartist": N_("original artist"),
+    "recordingdate": N_("recording date"),
+    "releasecountry": N_("release country"),
+    }
+
+del(N_)
